/***************************************************************************                      
    cfiles.h - display all files and dirs in the projectdir
                             -------------------                                         

    version              :                                   
    begin                : 9 Sept 1998                                        
    copyright            : (C) 1998 by Stefan Bartel                         
    email                : bartel@rz.uni-potsdam.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/
#ifndef KPROJECTDIRTREE_H
#define KPROJECTDIRTREE_H

#include <kapp.h>
#include <qwidget.h>
#include "ctreeview.h"
#include <kpopmenu.h>
#include <qstrlist.h>

//#include "cproject.h"
class CProject;
class TcpClient;

/**
  *display all files and dirs in the projectdir, a little filemanager
  @author Stefan Bartel 
  */
class CFiles : public CTreeView {
  Q_OBJECT 

public: 
  /** construtor */
  CFiles(QWidget*parent=0,const char* name=0);

  /** destructor */
  ~CFiles();

  /** starts the scan */
  void refresh(void);

protected: // Implementations of virtual methods.

    /** Get the current popupmenu. */
  virtual KPopupMenu *getCurrentPopup();
  /**  */

  /** */

  /**  */
  void getTcpConnection();

signals:
  /**
   * This signal is emitted when a file is selected with the left mousebutton 
   * or keyboard. It gives back the selected filename relative to the project 
   * directory. */
  void fileSelected(QString filename);
  /**
   * This signal is emitted when the "Add to Project" entry of the context menu
   * is selected. It gives back the selected filename relative to the project
   * directory. */
  void  saveFile(QString file);
  void  scloseFile(QString file);
  void addFileToProject(QString filename);
  /**
   * This signal is emitted when the "Remove from Project" entry of the 
   * context menu is selected. It gives back the selected filename relative to
   * the project directory. */
  void removeFileFromProject(QString filename);
  /**
   * This signal is emitted when the "File Properties..." entry of the context
   * menu is selected. It gives back the selected filename relative to the
   * project directory. */
  void showFileProperties(QString filename);

  void commitFileToVCS(QString file);
  void updateFileFromVCS(QString file);

  void commitDirToVCS(QString dir);
  void updateDirFromVCS(QString dir);

protected slots:
  /** emits signal "fileSelected" when it gets the selectionChanged-signal from itself */
  void slotSelectionChanged(QListViewItem* selection);
  void slotAddFileToProject();
  void slotRemoveFileFromProject();
  void slotDeleteFilePhys();
  void slotShowFileProperties();
  void slotAddToRepository();
  void slotRemoveFromRepository();
  void slotUpdate();
  void slotCommit();
	void slotSaveFile();
	void slotCloseFile();
	void slotLockFile();
	void slotOpenWorkingFile();
	void slotShowNonPrjFiles();
public slots:
	void slotCloseAll();

private: // Popupmenus
  KPopupMenu *popup;

private: // Private attributes
  QStrList filelist;
  int file_col;
	bool showNonPrjFiles;
  QListViewItem* pROItem;
  QListViewItem* pRWItem;
  QListViewItem* pRootItem;
//  CProject* project;
	QString projectDir;

private: // Private methods
  /** Adds the files from a certain directory to the view. */


  /** scans the project directory and fills the treelist */

  QString getRelFilename(QListViewItem* pItem);
  QString getFullFilename(QListViewItem* pItem);
  bool isInstalledFile(QString filename);

public: // Protected attributes
  /**  */
//  TcpClient* client;
  QString projectdir;

  void addROFile(QString file);
  void addRWFile(QString file);
  void addFile(QString file, int ck);
  void closeFile(QString file );
  int  existFile(QString file );
};
#endif





