/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "DialogCreateTable.h"

#define Inherited DialogCreateTableData

DialogCreateTable::DialogCreateTable(QWidget* parent, const char* name)
  : Inherited( parent, name )
  {
  setCaption( "Create table" );
  fieldList.setAutoDelete(true);
  }


DialogCreateTable::~DialogCreateTable()
  {
  }


void DialogCreateTable::addField(void)
  {
  DialogTableField dlg(this);
  if(dlg.exec())
    {
    TableField *item = dlg.result();
    if(getField(item->name) != 0)
      QMessageBox::critical(this, item->name, "Field name already exists.");
    else
    	{
    	fields->insertItem(item->name);
    	fieldList.append(item);
    	}
    }
  }

void DialogCreateTable::editField(void)
	{
	warning("mok");
	if(fields->currentItem() != -1)
	  {
		TableField *item = getField(fields->text(fields->currentItem()));
		if(!item)
			return;
		DialogTableField dlg(this);
		dlg.initFields(item);
		if(dlg.exec())	
			{
			int listIndex = fieldList.findRef(item);
			fieldList.removeRef(item);
			int listboxIndex = fields->currentItem();
			fields->removeItem(listboxIndex);
			TableField *newItem = dlg.result();
			fieldList.insert(listIndex, newItem);
			fields->insertItem(newItem->name, listboxIndex);
			}			
		}
	}

void DialogCreateTable::removeField(void)
	{
	if(fields->currentItem() != -1)
	  {
		TableField *item = getField(fields->text(fields->currentItem()));
		if(item)
			{
			fieldList.removeRef(item);
			fields->removeItem(fields->currentItem());
			}
		}
	}

TableField *DialogCreateTable::getField(QString fieldName)
	{
	for(unsigned int i=0; i<fieldList.count(); i++)
		if(fieldList.at(i)->name == fieldName)
			return fieldList.at(i);
	return 0;
	}
	
QString DialogCreateTable::getQuery(void)
	{
	QString res = "CREATE TABLE ";
	res += tableName->text();
	res += " (";
	for(unsigned int i=0; i<fieldList.count(); i++)
		{
		res += fieldList.at(i)->toSQL();
		if(i<fieldList.count()-1)
			res += ", ";
		}
	if(isTherePrimaryKey())
		{
		res += ", PRIMARY KEY(";
		bool first = true;
		for(unsigned int i=0; i<fieldList.count(); i++)
			if(fieldList.at(i)->isKey)
				{
				if(!first)
					res += ", ";
				else
					first = false;
				res += fieldList.at(i)->name;
				}
		res += ")";
		}
	res += " )";
	return res;
	}


bool DialogCreateTable::isTherePrimaryKey(void)
	{
	for(unsigned int i=0; i<fieldList.count(); i++)
		if(fieldList.at(i)->isKey)
			return true;
	return false;
	}
	
	









