/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "DialogModules.h"
#include "connectionmanager.h"
#include <kapp.h>

#define Inherited DialogModulesData

DialogModules::DialogModules(PluginManager *aPluginManager, QWidget* parent, const char *name)
	:	Inherited(parent, name)
	{
	pluginManager = aPluginManager;
	setCaption( i18n("Plugins") );
	connect(btnAdd, SIGNAL(clicked()), this, SLOT(addModule()));
	connect(btnInfo, SIGNAL(clicked()), this, SLOT(moduleInfo()));
	connect(btnRemove, SIGNAL(clicked()), this, SLOT(moduleRemove()));
	QStrList *names = pluginManager->getPluginNames();
	modules->insertStrList(names);
	delete names;
	}

	
DialogModules::~DialogModules()
	{
	}

	
void DialogModules::addModule(void)
	{
	KFileDialog *filedlg = new KFileDialog(0, 0, this, 0, true, false);
  if(filedlg->exec())
    {
    QString filename = filedlg->selectedFile();
    int id = pluginManager->registerPlugin(filename);
    if(id<0)
    	{
    	QString msg(512);
    	msg.sprintf("Error %d while loading plugin:\n%s", id, pluginManager->getErrorString());
    	QMessageBox::critical(this, "Error", msg);
    	return;
    	}	
		modules->insertItem(pluginManager->getPluginName(id));
		//warning("%d, %s", id, pluginManager->getPluginName(id));
    }
  delete filedlg;
	}


void DialogModules::moduleInfo(void)
	{
	QString name = modules->text(modules->currentItem());
	if(!name.isEmpty())
		{
		int id = pluginManager->getPluginByName(name);
		QString info = pluginManager->getPluginInfo(id);
		QString path = pluginManager->getPluginPath(id);
		//QMessageBox::information(this, name, info);		
		DialogModuleInfo *dlg = new DialogModuleInfo(this);
		dlg->setInfo(name, path, id, info);
		dlg->exec();
		delete dlg;
		}
	}

void DialogModules::moduleRemove(void)
	{
	QString name = modules->text(modules->currentItem());
	if(!name.isEmpty())
		{
    ConnectionManager *connectionManager = ConnectionManager::getInstance();
    // Check if the plugin is in use
    QString msg = i18n("This plugin can't be removed because it is used by the following connection(s):\n");
    bool used = false;
    for(unsigned int i=0; i<connectionManager->count(); i++)
    	{
    	Connection *connection = connectionManager->at(i);
    	if(connection->getPluginName() == name)
    		{
    		msg += "\n" + connection->getName();
    		used = true;
    		}
    	}
    if(used)
    	{
    	QMessageBox::warning(this, i18n("Remove plugin"), msg);
    	return;
    	}
		// Ok this plugin isn't used by any connection
		int id = pluginManager->getPluginByName(name);
		if(id < 0)
			{
			QMessageBox::warning(this, i18n("Remove plugin"), i18n("Can't find the plugin named ")+name);
			return;
			}
		int res = pluginManager->unregisterPlugin(id);
		if(res != 0)
			{
			QString msg;
			msg.sprintf(i18n("Error %d unregistering plugin %s"), res, (const char *)name);
			QMessageBox::warning(this, i18n("Remove plugin"), msg);
			return;
			}
		modules->removeItem(modules->currentItem());
		}
	}











