/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "DialogTableField.h"

#define Inherited DialogTableFieldData
#include <kapp.h>

// 0 : none
// 1 : int
// 2 : double
int oValidator[] = { 1, 1, 1, 1, 1,
										 1, 2, 2, 2, 2, 
										 2, 0, 0, 0, 0, 
										 0, 0, 0, 0, 0, 
										 0, 0, 0, 0 };

bool oLength[] = { true, true, true, true, true, 
									 true, true, true, true, true, 
									 true, true, true, false, false,
									 false, false, false, false, false, 
									 false, false, false, false };

bool oLengthRequired[] = { false, false, false, false, false,
	  	                   false, false, false, false, false,
      			             false, true, true, false, false, 
            			       false, false, false, false, false,
                  			 false, false, false, false };

bool oDecimals[] = { false, false, false, false, false, 
										false, true, true, true, true,
										true, false, false, false, false,
										false, false, false, false, false,
										false, false, false, false };

bool oUnsigned[] = { true, true, true, true, true, 
										 true, true, true, true, true,
										 true, false, false, false, false,
										 false, false, false, false, false,
										 false, false, false, false };

bool oZeroFill[] = { true, true, true, true, true, 
										 true, true, true, true, true,
										 true, false, false, false, false,
										 false, false, false, false, false,
										 false, false, false, false };

bool oBinary[] = { false, false, false, false, false,
                   false, false, false, false, false,
                   false, true, true, false, false, 
                   false, false, false, false, false,
                   false, false, false, false };

QString TableField::toSQL()
	{
	QString res = "";
	res += name + " ";
	res += type;
	if(specLength)
		{
		QString length; length.setNum(this->length);
		QString decimals; decimals.setNum(this->decimals);
		res += "(" + length;
		if(specDecimals)
		  res += ", "+decimals;
		res += ")";
		}
	res += " ";
	if(isUnsigned)
		res += "UNSIGNED ";
	if(binary)
		res += "BINARY ";
	if(zeroFill)
		res += "ZEROFILL ";
	if(canNull)
		res += "NULL ";
	else
	  res += "NOT NULL ";
	if(specDefaultValue)
		res += "DEFAULT '"+defaultValue+"' ";
	if(autoIncrement)
		res += "AUTO_INCREMENT ";
	//if(isKey)
		//res += "PRIMARY KEY";
	return res;
	}


DialogTableField::DialogTableField(QWidget* parent, const char* name)
  : Inherited( parent, name )
  {
  setCaption( i18n("Table column") );
  intValidator = new QIntValidator(this);
  doubleValidator = new QDoubleValidator(this);
  btngroup->setFrameStyle(QFrame::NoFrame);
  initFields();
  }

void DialogTableField::setPrimaryKeyEnabled(bool b)
	{
	isKey->setEnabled(b);
	autoIncrement->setChecked(false);
	autoIncrement->setEnabled(false);
	}	

void DialogTableField::initFields(void)
  {
  specLength->setChecked(false);
  length->setEnabled(false);
  length->setValue(1);
  specDefaultValue->setChecked(false);
  defaultValue->setEnabled(false);
  defaultValue->setText("");
  fieldName->setText("");
  fieldType->setCurrentItem(0);
  autoIncrement->setChecked(false);
  canNull->setChecked(true);
  cantNull->setChecked(false);
  typeChanged(0);
  }

void DialogTableField::setTypeCombo(QString value)
	{
	for(int i=0; i<fieldType->count(); i++)
		if(value == fieldType->text(i))
			fieldType->setCurrentItem(i);
	}

void DialogTableField::initFields(TableField *field)
	{
	fieldName->setText(field->name);
	setTypeCombo(field->type);
	specLength->setChecked(field->specLength);
	length->setEnabled(field->specLength);
	length->setValue(field->length);
	decimals->setEnabled(field->specDecimals);
	decimals->setValue(field->decimals);
	specDefaultValue->setChecked(field->specDefaultValue);
	defaultValue->setText(field->defaultValue);
	autoIncrement->setChecked(field->autoIncrement);
	canNull->setChecked(field->canNull);
	cantNull->setChecked(!field->canNull);
	binary->setChecked(field->binary);
	zeroFill->setChecked(field->zeroFill);
	isUnsigned->setChecked(field->isUnsigned);
	isKey->setChecked(field->isKey);
	typeChanged(fieldType->currentItem());
	specLengthChanged();
	specDefaultValueChanged();
	}

DialogTableField::~DialogTableField()
  {
  }

void DialogTableField::ok(void)
	{
	if(QString(fieldName->text()).isEmpty())
	  QMessageBox::critical(this, i18n("Table column"), i18n("Please give a name to this column"));
	else
		done(1);
	}

void DialogTableField::typeChanged(int i)
  {
  specLength->setEnabled(oLength[i]);
  if(oLengthRequired[i])
  	{
  	specLength->setChecked(true);
  	specLength->setEnabled(false);
  	length->setEnabled(true);
  	}
  specLengthChanged();
  isUnsigned->setEnabled(oUnsigned[i]);
  zeroFill->setEnabled(oZeroFill[i]);
  binary->setEnabled(oBinary[i]);
  if(oValidator[i] == 0)
    defaultValue->setValidator(0);
  else if(oValidator[i] == 1)
  	defaultValue->setValidator(intValidator);
  else if(oValidator[i] == 2)
  	defaultValue->setValidator(doubleValidator);
	//defaultValue->setText("");
  }

void DialogTableField::specLengthChanged(void)
  {
  length->setEnabled(specLength->isChecked());
	if(oDecimals[fieldType->currentItem()])
  	decimals->setEnabled(specLength->isChecked());
  else
  	decimals->setEnabled(false);
  }

void DialogTableField::specDefaultValueChanged(void)
  {
  defaultValue->setEnabled(specDefaultValue->isChecked());
  }

void DialogTableField::isKeyChanged(bool b)
	{
	if(b)
		{
		canNull->setChecked(!b);
		cantNull->setChecked(b);
		}
	canNull->setEnabled(!b);
	cantNull->setEnabled(!b);
	}

TableField *DialogTableField::result(void)
  {
  TableField *res = new TableField;
  res->name = fieldName->text();
  res->type = fieldType->currentText();
  res->specLength = length->isEnabled();
  res->length = length->value();
  res->specDecimals = decimals->isEnabled();
  res->decimals = decimals->value();
  res->specDefaultValue = specDefaultValue->isChecked();
  res->defaultValue = defaultValue->text();
  res->autoIncrement = autoIncrement->isChecked();
  res->canNull = canNull->isChecked();
  res->binary = binary->isEnabled() && binary->isChecked();
  res->zeroFill = zeroFill->isEnabled() && zeroFill->isChecked();
  res->isUnsigned = isUnsigned->isEnabled() && isUnsigned->isChecked();
  res->isKey = isKey->isChecked();
  return res;
  }







