/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dlgdepropcombo.h"
#include "feframe.h"
#include <qmessagebox.h>
#include <kapp.h>


DlgDePropCombo::DlgDePropCombo(QWidget *parent, FeCombo *c)
	: QDialog(parent, 0, true)
	{
	initDialog();
	connect(btnOk, SIGNAL(clicked()), this, SLOT(slotOk()));
	connect(btnCancel, SIGNAL(clicked()), this, SLOT(reject()));
	connect(btnHelp, SIGNAL(clicked()), this, SLOT(help()));
	connect(btnAddChoice, SIGNAL(clicked()), this, SLOT(addChoice()));
	connect(btnRemoveChoice, SIGNAL(clicked()), this, SLOT(removeChoice()));
	connect(radioChoicesList, SIGNAL(toggled(bool)), this, SLOT(choicesList(bool)));
	connect(radioChoicesQuery, SIGNAL(toggled(bool)), this, SLOT(choicesQuery(bool)));
	groupChoices = new QButtonGroup(this);
	groupChoices->hide();
	groupChoices->insert(radioChoicesList);
	groupChoices->insert(radioChoicesQuery);	
	//
	combo = c;
	load();
	}

DlgDePropCombo::~DlgDePropCombo(void)
	{
	}
	
void DlgDePropCombo::slotOk()
	{
	FeFrame *frame = (FeFrame *)combo->parentWidget();
	if(frame->nameExists(editName->text(), combo))
		QMessageBox::warning(this, i18n("Name clash"), i18n("This name is already used by another widget\nPlease choose another one."));
	else
		accept();
	}

void DlgDePropCombo::help(void)
	{
	KApplication::getKApplication()->invokeHTMLHelp("kmysql/index.html", "WidgetCombo");
	}
			
void DlgDePropCombo::load(void)
	{
	editName->setText(combo->name());
	editQuery->setText(combo->choicesQuery());
	listChoices->insertStrList(combo->choices());
	if(combo->choicesOrigin() == FeCombo::List)
		{
		choicesList(true);
		choicesQuery(false);
		radioChoicesList->setChecked(true);
		}
	else if(combo->choicesOrigin() == FeCombo::Query)
		{
		choicesList(false);
		choicesQuery(true);
		radioChoicesQuery->setChecked(true);
		}
	}

	
void DlgDePropCombo::save(void)
	{
	combo->setName(editName->text());
	combo->setChoicesQuery(editQuery->text());
	combo->choices()->clear();
	for(unsigned int i=0; i<listChoices->count(); i++)
		combo->choices()->append(listChoices->text(i));
	if(radioChoicesList->isChecked())
		combo->setChoicesOrigin(FeCombo::List);
	else if(radioChoicesQuery->isChecked())
		combo->setChoicesOrigin(FeCombo::Query);
	}


void DlgDePropCombo::addChoice()
	{
	QString choice = editChoice->text();
	if(choice.isEmpty())
		return;
	listChoices->insertItem(choice);
	editChoice->clear();
	}
	
void DlgDePropCombo::removeChoice()
	{
	int i = listChoices->currentItem();
	if(i != -1)
		listChoices->removeItem(i);
	}
	
void DlgDePropCombo::choicesList(bool b)
	{
	btnAddChoice->setEnabled(b);
	btnRemoveChoice->setEnabled(b);
	listChoices->setEnabled(b);
	editChoice->setEnabled(b);
	}
	
void DlgDePropCombo::choicesQuery(bool b)
	{
	editQuery->setEnabled(b);
	}

