/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dlgfepropbutton.h"
#include <kapp.h>
#include "feframe.h"
#include <qobjectlist.h>
#include <qmessagebox.h>
#include <kapp.h>


DlgFePropButton::DlgFePropButton(QWidget *parent, FeButton *btn)
	: QDialog(parent, 0, true)
	{
	initDialog();
	connect(btnOk, SIGNAL(clicked()), this, SLOT(slotOk()));
	connect(btnCancel, SIGNAL(clicked()), this, SLOT(reject()));
	connect(btnHelp, SIGNAL(clicked()), this, SLOT(help()));
	comboFunction->insertItem(i18n("Submit"));
	comboFunction->insertItem(i18n("Cancel"));
	comboFunction->insertItem(i18n("Help"));
	//
	button = btn;
	load();
	}

DlgFePropButton::~DlgFePropButton(void)
	{
	}
	
void DlgFePropButton::help(void)
	{
	KApplication::getKApplication()->invokeHTMLHelp("kmysql/index.html", "WidgetButton");
	}
	
void DlgFePropButton::slotOk()
	{
	FeFrame *frame = (FeFrame *)button->parentWidget();
	if(frame->nameExists(editName->text(), button))
		QMessageBox::warning(this, i18n("Name clash"), i18n("This name is already used by another widget\nPlease choose another one."));
	else
		accept();
	}
	
void DlgFePropButton::save(void)
	{
	button->setName(editName->text());
	button->setText(editText->text());
	switch(comboFunction->currentItem())
		{
		case 0:
			button->setFunction(FeButton::Submit);
			break;
		case 1:
			button->setFunction(FeButton::Cancel);
			break;
		case 2:
			button->setFunction(FeButton::Help);
			break;
		}
	button->setDefault(cbDefault->isChecked());
	}
	
void DlgFePropButton::load(void)
	{
	editName->setText(button->name());
	editText->setText(button->text());
	switch(button->function())
		{
		case FeButton::Submit:
			comboFunction->setCurrentItem(0);
			break;
		case FeButton::Cancel:
			comboFunction->setCurrentItem(1);
			break;
		case FeButton::Help:
			comboFunction->setCurrentItem(2);
			break;
		}
	cbDefault->setChecked(button->isDefault());
	}





