/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "editabletablistbox.h"

EditableTabListBoxColumn::EditableTabListBoxColumn(KTabListBox *parent, int w)
	: KTabListBoxColumn(parent, w)
	{
	}
	
void EditableTabListBoxColumn::paintCell(QPainter *paint, int row, const QString &string, bool marked)
	{
	emit sigPaintCell();
	KTabListBoxColumn::paintCell(paint, row, string, marked);
	}

EditableTabListBox::EditableTabListBox(QWidget *parent, const char *name)
	: KTabListBox(parent, name), currentCellEditor(0), editing(false), editable(false)
	{
	connect(this, SIGNAL(selected(int, int)), this, SLOT(slotSelected(int, int)));
	connect(this, SIGNAL(highlighted(int, int)), this, SLOT(slotHighlighted(int, int)));
	}
		
EditableTabListBox::~EditableTabListBox(void)
	{
	}

void EditableTabListBox::setEditable(bool b)
	{
	editable = b;
	}

KTabListBoxColumn *EditableTabListBox::newKTabListBoxColumn(void)
	{
	EditableTabListBoxColumn *res = new EditableTabListBoxColumn(this);
	connect(res, SIGNAL(sigPaintCell()), this, SLOT(slotPaintCell()));
	return res;
	}	

void EditableTabListBox::slotPaintCell()
	{
	if(editable && editing)
		updateCellEditor();
	}
	
void EditableTabListBox::updateCellEditor()
	{
	if(!currentCellEditor)
		return;
	int x, y, w, h;
	bool b = colXPos(editCol, &x);
	rowYPos(editRow, &y);
	w = cellWidth(editCol);
	h = cellHeight(editRow);
	y += h+3;
	if(b)
		{
		currentCellEditor->show();
		currentCellEditor->setGeometry(x, y, w, h);
		}
	else
		currentCellEditor->hide();
	}
	


void EditableTabListBox::slotSelected(int row, int col)
	{
	if(editing)
		if(row != editRow || col != editCol)
			stopEditCell(false);
	if(editable)
		editCell(row, col);
	}
	
void EditableTabListBox::slotHighlighted(int row, int col)
	{
	if(editing)
		if(row != editRow || col != editCol)
			stopEditCell(false);
	}

	
void EditableTabListBox::putCellEditor(int row, int col)
	{
	int x, y, w, h;
	colXPos(col, &x);
	rowYPos(row, &y);
	w = cellWidth(col);
	h = cellHeight(row);
	y += h+3;
	CellEdit *edit = new CellEdit(this);
	edit->setGeometry(x, y, w, h);
	edit->setText(QString(text(row, col)).stripWhiteSpace());
	edit->setFocus();
	edit->show();
	currentCellEditor = edit;
	}
	
void EditableTabListBox::editCell(int row, int col)
	{
	editRow = row;
	editCol = col;
	setCurrentItem(row);
	putCellEditor(row, col);
	oldValue = QString(text(row, col)).stripWhiteSpace();
	connect(currentCellEditor, SIGNAL(done(bool)), this, SLOT(stopEditCell(bool)));
	editing = true;
	}	
		
void EditableTabListBox::stopEditCell(bool save)
	{
	disconnect(currentCellEditor);
	currentCellEditor->hide();
	newValue = currentCellEditor->text();
	delete currentCellEditor;
	currentCellEditor = 0;
	if(save)
		{
		if(validateCellChange(editRow, editCol, oldValue, newValue))
			{
			changeItemPart(newValue, editRow, editCol);
			emit cellChanged(editRow, editCol, oldValue, newValue);
			}
		}
	editing = false;
	}

bool EditableTabListBox::validateCellChange(int, int, const char *, const char *)
	{
	return true;
	}	







