/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "febutton.h"
#include "dlgfepropbutton.h"
#include "utils.h"

FeButton::FeButton(QWidget *parent)
	: FeButtonInherited(parent)
	{
	}
	
	
bool FeButton::setOptions(QString str)
	{
	Utils::Option option;
	if(!option.fromString(str))
		return false;
	if(option.getName() == "options")
		{
		if(option.getParamCount() != 3)
			{
			warning("incorrect button params count in options");
			return false;
			}
		setText(option.getParam(0));
		QString actionType = option.getParam(1);
		if(actionType == "submit")
			setFunction(Submit);
		else if(actionType == "cancel")
			setFunction(Cancel);
		else if(actionType == "help")
			setFunction(Help);
		QString isDefault = option.getParam(2);
		setDefault(isDefault=="true"?true:false);	
		}
	else
		{
		warning("unknown button option: '%s'", (const char *)option.getName());
		return false;
		}
	return true;
	}
	
bool FeButton::writeDatabase(ISqlServer *sql, QString formId)
	{
	if(!sql)
		return false;
	Utils::Option options;
	options.setName("options");
	options.addParam(text());
	switch(function())
		{
		case Submit: options.addParam("submit"); break;
		case Cancel: options.addParam("cancel"); break;
		case Help: options.addParam("help"); break;
		default: return false;
		}
	if(isDefault())
		options.addParam("true");
	else
		options.addParam("false");
	QString extra = Utils::insertQuotes(options.toString());
	QString query(16000);
	query.sprintf("INSERT INTO __KmysqlFields (FormRef, Name, Type, X, Y, Width, Height, Extra) VALUES (%s, '%s', '%s', %d, %d, %d, %d, '%s')",
								(const char *)formId, (const char *)Utils::insertQuotes(name()), "button", x(), y(), width(), height(), (const char *)extra);
  QueryResult res = sql->query(query);
  if(res.type == QueryResult::Error)
  	return false;
  return true;
	}


void FeButton::showProperties()
	{
	DlgFePropButton dlg(this, this);
	if(dlg.exec())
		dlg.save();
	}	
	
























