/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FEWIDGET_H
#define FEWIDGET_H

#include <qpopupmenu.h>
#include <qpainter.h>
#include "fegrid.h"
#include "fewidgetmenu.h"

template<class BaseClass>
class FeWidget : public BaseClass
	{
	public:
	
		FeWidget(QWidget *parent, bool isFrame = false) : BaseClass(parent), cornerSize(7)
			{
			moving = false;
			resizing = false;
			popup = new FeWidgetMenu(this, isFrame);
			}
			
	protected:
	
		virtual bool event(QEvent *e)
			{
			if(e->type() == Event_User_Properties)
				{
				showProperties();
				return true;
				}
			else
				{
				return BaseClass::event(e);
				}
			}
	
		virtual void mousePressEvent(QMouseEvent *e)
			{
			if(e->button() == LeftButton)
				{	
				if(isOnCorner(e->pos()))
					{
					resizing = true;
					clickedPos = e->pos();
					}		
				else
					{
					moving = true;
					clickedPos = e->pos();
					}
				}
			else if(e->button() == RightButton)
				popup->popup(mapToGlobal(e->pos()));
			}
			
		virtual void mouseReleaseEvent(QMouseEvent *e)
			{
			if(e->state() == LeftButton)
				moving = resizing = false;
			}
			
		virtual void mouseMoveEvent(QMouseEvent *e)
			{
			if(e->state() == LeftButton)
				{	
				if(moving)
					move(FeGrid::manage(pos()+e->pos()-clickedPos));
				else if(resizing)
					{
					QSize newSize(e->pos().x(), e->pos().y());
		 			resize(FeGrid::manage(newSize));
		  		}
				}
			}
			
		virtual void paintEvent(QPaintEvent *e)
			{
			BaseClass::paintEvent(e);
			QPainter p;
			p.begin(this);
			p.setPen(black);
			p.setBrush(black);
			p.drawRect(width()-cornerSize, height()-cornerSize, width(), height());
			p.end();
			}
			
		virtual bool isOnCorner(QPoint pos)
			{
			QPoint top(width()-cornerSize, height()-cornerSize);
			QPoint bottom(width(), height());
			QRect rect(top, bottom);
			return rect.contains(pos);
			}
			
		virtual void showProperties(void) = 0;
			
	private:
		bool moving, resizing;
		QPoint clickedPos;
		const int cornerSize;
		QPopupMenu *popup;
	};

#endif










