/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "formdialog.h"
#include "utils.h"
#include "qmessagebox.h"
#include <qobjectlist.h>

FormDialog::FormDialog(QWidget* parent)
  : QDialog(parent, 0, true)
  {
  }

FormDialog::~FormDialog(void)
  {
  }

void FormDialog::slotOk(void)
  {
  done(1);
  }

void FormDialog::slotCancel(void)
  {
  done(0);
  }

void FormDialog::slotHelp(void)
	{
	QMessageBox::information(this, "Help", help);
	}

int FormDialog::load(ISqlServer *sql, QString formName)	
	{
	bool b;
	if(!sql)
		return 1;
	QString request = "SELECT Ref,Name,Query,Help,Width,Height FROM __KmysqlForms WHERE Name='"+formName+"'";
	warning("%s", (const char *)request);
	QueryResult res = sql->query(request);
	if(res.type == QueryResult::Error)
		return 2;
	QString fields = res.result.at(0);
	QString fieldFormRef = Utils::subStringAt(fields, '\t', 0);
	QString fieldName = Utils::subStringAt(fields, '\t', 1);
	QString fieldQuery = Utils::subStringAt(fields, '\t', 2);
	QString fieldHelp = Utils::subStringAt(fields, '\t', 3);
	QString fieldWidth = Utils::subStringAt(fields, '\t', 4);	
	QString fieldHeight = Utils::subStringAt(fields, '\t', 5);		
	query = fieldQuery;	
	setCaption(formName);
	help = fieldHelp;
	setFixedSize(fieldWidth.toInt(&b), fieldHeight.toInt(&b));
	// Les champs
	request = "SELECT Name,Type,X,Y,Width,Height,Extra FROM __KmysqlFields WHERE FormRef="+fieldFormRef;
	warning("%s", (const char *)request);
	res = sql->query(request);
	if(res.type == QueryResult::Error)
		return 3;
	for(unsigned int i=0; i<res.result.count(); i++)
		{
		QString fields = res.result.at(i);
		QString fieldName = Utils::subStringAt(fields, '\t', 0);
		QString fieldType = Utils::subStringAt(fields, '\t', 1);
		QString fieldX = Utils::subStringAt(fields, '\t', 2);
		QString fieldY = Utils::subStringAt(fields, '\t', 3);
		QString fieldWidth = Utils::subStringAt(fields, '\t', 4);
		QString fieldHeight = Utils::subStringAt(fields, '\t', 5);
		QString fieldExtra = Utils::subStringAt(fields, '\t', 6);
		if(fieldType=="text")
			{
			QLineEdit *widget = new QLineEdit(this);
			widget->setName(fieldName);
			widget->setGeometry(fieldX.toInt(&b), fieldY.toInt(&b), fieldWidth.toInt(&b), fieldHeight.toInt(&b));
			Utils::Option option;
			if(!option.fromString(fieldExtra))
				return 42;
			if(option.getName() == "options")
				{
				QString defaultValue = option.getParam(0);
				widget->setText(defaultValue);
				}
			widget->show();
			}
		else if(fieldType=="checkbox")
			{
			QMyCheckBox *widget = new QMyCheckBox(this);
			widget->setName(fieldName);
			widget->setGeometry(fieldX.toInt(&b), fieldY.toInt(&b), fieldWidth.toInt(&b), fieldHeight.toInt(&b));
			Utils::Option option;
			if(!option.fromString(fieldExtra))
				return 53;
			if(option.getName() == "options")
				{
				QString text = option.getParam(0);
				widget->setText(text);
				QString initial = option.getParam(1);
				widget->setChecked(initial == "On");
				widget->checkedValue = option.getParam(2);
				widget->uncheckedValue = option.getParam(3);
				}
			widget->show();
			}
		else if(fieldType=="combo")
			{
			QMyComboBox *widget = new QMyComboBox(this);
			widget->setName(fieldName);
			widget->setGeometry(fieldX.toInt(&b), fieldY.toInt(&b), fieldWidth.toInt(&b), fieldHeight.toInt(&b));
			Utils::Option option;
			if(!option.fromString(fieldExtra))
				return 4;
			if(option.getName() == "options")
				{
				QString mode = option.getParam(0);
				QString query = option.getParam(1);
				if(mode == "query")
					{
					QueryResult res = sql->query(query);
					if(res.type!=QueryResult::Set)
						return -1;
					if(res.numColumns < 1 || res.numColumns > 2)
						{
						QMessageBox::warning(this, "Combo box query", "A combo-box query must select one or two columns!");
						return -1;
						}
					if(res.numColumns == 1)
						{
						for(unsigned int i=0; i<res.result.count(); i++)	
							{
							widget->insertItem(res.result.at(i));
							widget->values.append(res.result.at(i));
							}
						}
					else if(res.numColumns == 2)
						{
						for(unsigned int i=0; i<res.result.count(); i++)	
							{
							widget->insertItem(Utils::subStringAt(res.result.at(i), '\t', 0));
							widget->values.append(Utils::subStringAt(res.result.at(i), '\t', 1));
							}
						}							
					}
				else
					{
					for(unsigned int i = 2; i < option.getParamCount(); i++)
						{
						widget->insertItem(option.getParam(i));
						widget->values.append(option.getParam(i));
						}
					}
				}
			widget->show();
			}
		else if(fieldType=="button")
			{
			QPushButton *widget = new QPushButton(this);
			widget->setName(fieldName);
			widget->setGeometry(fieldX.toInt(&b), fieldY.toInt(&b), fieldWidth.toInt(&b), fieldHeight.toInt(&b));
			Utils::Option option;
			if(!option.fromString(fieldExtra))
				return 5;
			if(option.getName() == "options")
				{
				if(option.getParamCount() != 3)
					return 6;
				widget->setText(option.getParam(0));
				QString actionType = option.getParam(1);
				if(actionType == "submit")
					connect(widget, SIGNAL(clicked()), this, SLOT(slotOk()));
				else if(actionType == "cancel")
					connect(widget, SIGNAL(clicked()), this, SLOT(slotCancel()));
				else if(actionType == "help")
					connect(widget, SIGNAL(clicked()), this, SLOT(slotHelp()));
				else
					return 7;
				QString isDefault = option.getParam(2);
				widget->setDefault(isDefault=="true"?true:false);
				}
			widget->show();
			}
		else if(fieldType=="label")
			{
			QLabel *widget = new QLabel(this);
			widget->setName(fieldName);
			widget->setGeometry(fieldX.toInt(&b), fieldY.toInt(&b), fieldWidth.toInt(&b), fieldHeight.toInt(&b));
			Utils::Option option;
			if(!option.fromString(fieldExtra))
				return 87;
			if(option.getName() == "options")
				{
				if(option.getParamCount() != 1)
					return 9;
				widget->setText(option.getParam(0));
				}
			widget->show();
			}
		else if(fieldType=="date")
			{
			KMyDatePicker *widget = new KMyDatePicker(this);
			widget->setName(fieldName);
			widget->setGeometry(fieldX.toInt(&b), fieldY.toInt(&b), fieldWidth.toInt(&b), fieldHeight.toInt(&b));
			widget->show();
			}
		else
			return 10;
		}		
	return 0;
	}

QString FormDialog::makeQuery(void)
  {
  if(query.isEmpty())
    return "";
  QString res = "";
  unsigned int len = query.length();
  for(unsigned int i=0; i<len; i++)
    {
    if(query[i] != substChar)
      {
      res += query[i];
      }
    else
      {
      QString fieldName = "";
      i++;
      while(i<len && query[i] != substChar)
			fieldName += query[i++];
      res += Utils::insertQuotes(getValue(fieldName));
      }
    }
  return res;
  }

QString FormDialog::getValue(QString fieldName)
  {
	if(!children())
		return "";
  QObjectListIt it(*children());
  QObject *obj;
	while ((obj=it.current()) != 0)
		{
		++it;
		QWidget *widget = (QWidget *)obj;
		if(widget->name() == fieldName)
			return widgetValue(widget);
		}
	return "";
  }

QString FormDialog::widgetValue(QWidget *widget)
	{
	QString className = widget->className();
	if(className == "QPushButton")
		{
		return ((QPushButton *)widget)->text();
		}
	else if(className == "QLabel")
		{
		return ((QLabel *)widget)->text();
		}
	else if(className == "QLineEdit")
		{
		return ((QLineEdit *)widget)->text();
		}
	else if(className == "QMyComboBox")
		{
		QMyComboBox *combo = (QMyComboBox *)widget;
		return combo->values.at(combo->currentItem());
		}
	else if(className == "QMyCheckBox")
		{
		QMyCheckBox *cb = (QMyCheckBox *)widget;
		if(cb->isChecked())
			return cb->checkedValue;
		else
			return cb->uncheckedValue;
		}
	else if(className == "KMyDatePicker")
		{
		KMyDatePicker *dp = (KMyDatePicker *)widget;
		QDate date = dp->selectedDate();
		QString res;
		res.sprintf("%d-%d-%d", date.year(), date.month(), date.day());
		return res;
		}
	else
		return "";
	}

















