/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kapp.h>
#include <qobjectlist.h>
#include "formeditorview.h"
#include "utils.h"
#include "mainwindow.h"


FormEditorView::FormEditorView(MainWindow *parent)
  : View(parent)
  {
  setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  setBackgroundColor(darkGray);
	loadPref();
	initWidgets();
  initMenus();
  initToolBars();
  initConnects();
  }

void FormEditorView::initWidgets(void)
	{
	frame = new FeFrame(this);
  frame->setFrameStyle(QFrame::WinPanel | QFrame::Raised);
  frame->setLineWidth(2);
  frame->setGeometry(50, 50, 200, 200);
	}
	
void FormEditorView::initMenus(void)
	{
	KIconLoader *iconLoader = KApplication::getKApplication()->getIconLoader();
	menu = new KMenuBar(mainWindow());
  QPopupMenu *menuEdit = new QPopupMenu();
  menuEdit->insertItem(iconLoader->loadIcon("configure.xpm"), i18n("&Options..."), this, SLOT(config()), CTRL+Key_O);
	QPopupMenu *menuForm = new QPopupMenu();
	menuForm->insertItem(iconLoader->loadIcon("filedel.xpm"), i18n("&Cancel"), mainWindow()->mainView(), SLOT(slotFormCancel()), CTRL+Key_Z);
	menuForm->insertItem(iconLoader->loadIcon("filefloppy.xpm"), i18n("&Save"), mainWindow()->mainView(), SLOT(slotFormSave()), CTRL+Key_S);
  QPopupMenu *menuItems = new QPopupMenu();
  menuItems->insertItem(iconLoader->loadIcon("febutton.xpm"), i18n("New &button"), this, SLOT(newButton()), CTRL+Key_B);
  menuItems->insertItem(iconLoader->loadIcon("felabel.xpm"), i18n("New &label"), this, SLOT(newLabel()), CTRL+Key_L);
  menuItems->insertItem(iconLoader->loadIcon("feedit.xpm"), i18n("New &line-edit"), this, SLOT(newEdit()), CTRL+Key_E);
  menuItems->insertItem(iconLoader->loadIcon("fecombo.xpm"), i18n("New &combo-box"), this, SLOT(newCombo()), CTRL+Key_M);
  menuItems->insertItem(iconLoader->loadIcon("fecheckbox.xpm"), i18n("New &check-box"), this, SLOT(newCheckBox()), CTRL+Key_H);
  menuItems->insertItem(iconLoader->loadIcon("fedate.xpm"), i18n("New &date picker"), this, SLOT(newDate()), CTRL+Key_D);
  menu->insertItem(i18n("&Form"), menuForm);
  menu->insertItem(i18n("&Edit"), menuEdit);
  menu->insertItem(i18n("&Items"), menuItems);
	}
	
void FormEditorView::initToolBars(void)
	{
	// Main toolbar
  KToolBar *tbMain = new KToolBar(mainWindow());
  KIconLoader *iconLoader = KApplication::getKApplication()->getIconLoader();
	QPixmap pixFe1 = iconLoader->loadIcon(QString("filefloppy.xpm"));
  tbMain->insertButton(pixFe1, 1, SIGNAL(clicked()), mainWindow()->mainView(), SLOT(slotFormSave()), true, "Save form");
  QPixmap pixFe2 = iconLoader->loadIcon(QString("filedel.xpm"));
  tbMain->insertButton(pixFe2, 2, SIGNAL(clicked()), mainWindow()->mainView(), SLOT(slotFormCancel()), true, "Cancel");
  tbMain->insertSeparator();
  QPixmap pixFe7 = iconLoader->loadIcon(QString("configure.xpm"));
  tbMain->insertButton(pixFe7, 3, SIGNAL(clicked()), this, SLOT(config()), true, "Configure form editor");
  // Items toolbal
	KToolBar *tbItems = new KToolBar(mainWindow());
	tbItems->setBarPos(KToolBar::Left);
	QPixmap pixFe3 = iconLoader->loadIcon(QString("febutton.xpm"));
  tbItems->insertButton(pixFe3, 1, SIGNAL(clicked()), this, SLOT(newButton()), true, "Add button");
  QPixmap pixFe4 = iconLoader->loadIcon(QString("felabel.xpm"));
  tbItems->insertButton(pixFe4, 2, SIGNAL(clicked()), this, SLOT(newLabel()), true, "Add label");
  QPixmap pixFe5 = iconLoader->loadIcon(QString("feedit.xpm"));
  tbItems->insertButton(pixFe5, 3, SIGNAL(clicked()), this, SLOT(newEdit()), true, "Add edit box");
  QPixmap pixFe6 = iconLoader->loadIcon(QString("fecombo.xpm"));
  tbItems->insertButton(pixFe6, 4, SIGNAL(clicked()), this, SLOT(newCombo()), true, "Add combo box");
	QPixmap pixFe8 = iconLoader->loadIcon(QString("fecheckbox.xpm"));
  tbItems->insertButton(pixFe8, 5, SIGNAL(clicked()), this, SLOT(newCheckBox()), true, "Add check-box");
  QPixmap pixFe9 = iconLoader->loadIcon(QString("fedate.xpm"));
  tbItems->insertButton(pixFe9, 6, SIGNAL(clicked()), this, SLOT(newDate()), true, "Add date-picker");
  tbIdMain = mainWindow()->addToolBar(tbMain);
	tbIdItems = mainWindow()->addToolBar(tbItems);
	}
	
void FormEditorView::initConnects(void)
	{
	
	}

QString FormEditorView::newName(void)
	{
	QString res;
	do
		{
		res.sprintf("Item%d", widgetCount++);
		} while(frame->nameExists(res));
	return res;
	}

void FormEditorView::newButton()
	{
	FeButton *btn = new FeButton(frame);
	btn->setGeometry(50, 50, 50, 30);	
	btn->setText(i18n("Button"));
	btn->setName(newName());
	btn->show();
	}
	
void FormEditorView::newLabel()
	{
	FeLabel *lbl = new FeLabel(frame);
	lbl->setGeometry(50, 50, 50, 30);	
	lbl->setText(i18n("Label"));
	lbl->setName(newName());
	lbl->show();
	}

	
void FormEditorView::newEdit()
	{
	FeEdit *edit = new FeEdit(frame);
	edit->setGeometry(50, 50, 50, 30);	
	edit->setName(newName());
	edit->show();
	}

void FormEditorView::newCombo()
	{
	FeCombo *combo = new FeCombo(frame);
	combo->setGeometry(50, 50, 50, 30);	
	combo->setName(newName());
	combo->show();
	}

void FormEditorView::newCheckBox()
	{
	FeCheckBox *cb = new FeCheckBox(frame);
	cb->setGeometry(50, 50, 50, 30);	
	cb->setName(newName());
	cb->setText(i18n("Check-box"));
	cb->show();
	}

void FormEditorView::newDate()
	{
	FeDate *date = new FeDate(frame);
	date->setGeometry(50, 50, 100, 100);	
	date->setName(newName());
	date->show();
	}	

void FormEditorView::formProperties()
	{
	}

void FormEditorView::formSave()
	{
	}

void FormEditorView::formCancel()
	{
	}	


void FormEditorView::config()
	{
	DlgFeConfig dlg(this);
	if(dlg.exec())
		{
		dlg.save();
		savePref();
		frame->repaint();
		}
	}


void FormEditorView::savePref(void)
	{
	KConfig *config = KApplication::getKApplication()->getConfig();
  config->setGroup("FormEditor");
  config->writeEntry("GridSize", FeGrid::step());
  config->writeEntry("Visible", FeGrid::isVisible());	
  config->writeEntry("Fit", FeGrid::isFit());	
	}

void FormEditorView::loadPref(void)
	{
	KConfig *config = KApplication::getKApplication()->getConfig();
  config->setGroup("FormEditor");
  FeGrid::setStep(config->readNumEntry("GridSize", 10));
  FeGrid::setVisible(config->readBoolEntry("Visible", true));
	FeGrid::setFit(config->readBoolEntry("Fit", true));
	}
	

void FormEditorView::clear(void)
	{
	widgetCount = 0;
	frame->setTitle("");
	frame->setQuery("");
	frame->setHelp("");
	const QObjectList *children = frame->children();
	if(children)
		{
	  QObjectListIt it(*children);
		QObject *obj;
		while ((obj=it.current()) != 0)
			{
			++it;
			delete obj;
			}
		}
	}	


int FormEditorView::load(ISqlServer *sql, QString formName)
	{
	bool b;
	if(!sql)
		return 1;
	clear();
	QString request = "SELECT Ref,Name,Query,Help,Width,Height FROM __KmysqlForms WHERE Name='"+formName+"'";
	warning("%s", (const char *)request);
	QueryResult res = sql->query(request);
	if(res.type == QueryResult::Error)
		return 2;
	QString fields = res.result.at(0);
	QString fieldFormRef = Utils::subStringAt(fields, '\t', 0);
	QString fieldName = Utils::subStringAt(fields, '\t', 1);
	QString fieldQuery = Utils::subStringAt(fields, '\t', 2);
	QString fieldHelp = Utils::subStringAt(fields, '\t', 3);
	QString fieldWidth = Utils::subStringAt(fields, '\t', 4);	
	QString fieldHeight = Utils::subStringAt(fields, '\t', 5);		
	frame->setQuery(fieldQuery);	
	frame->setTitle(formName);
	frame->setHelp(fieldHelp);
	frame->resize(fieldWidth.toInt(&b), fieldHeight.toInt(&b));
	// Les champs
	request = "SELECT Name,Type,X,Y,Width,Height,Extra FROM __KmysqlFields WHERE FormRef="+fieldFormRef;
	warning("%s", (const char *)request);
	res = sql->query(request);
	if(res.type == QueryResult::Error)
		return 3;
	for(unsigned int i=0; i<res.result.count(); i++)
		{
		QString fields = res.result.at(i);
		QString fieldName = Utils::subStringAt(fields, '\t', 0);
		QString fieldType = Utils::subStringAt(fields, '\t', 1);
		QString fieldX = Utils::subStringAt(fields, '\t', 2);
		QString fieldY = Utils::subStringAt(fields, '\t', 3);
		QString fieldWidth = Utils::subStringAt(fields, '\t', 4);
		QString fieldHeight = Utils::subStringAt(fields, '\t', 5);
		QString fieldExtra = Utils::subStringAt(fields, '\t', 6);
		if(fieldType=="text")
			{
			FeEdit *widget = new FeEdit(frame);
			widget->setName(fieldName);
			widget->setGeometry(fieldX.toInt(&b), fieldY.toInt(&b), fieldWidth.toInt(&b), fieldHeight.toInt(&b));
			if(!widget->setOptions(fieldExtra))
				return 4;
			widget->show();
			}
		else if(fieldType=="combo")
			{
			FeCombo *widget = new FeCombo(frame);
			widget->setName(fieldName);
			widget->setGeometry(fieldX.toInt(&b), fieldY.toInt(&b), fieldWidth.toInt(&b), fieldHeight.toInt(&b));
			if(!widget->setOptions(fieldExtra))
				return 5;
			widget->show();
			}
		else if(fieldType=="button")
			{
			FeButton *widget = new FeButton(frame);
			widget->setName(fieldName);
			widget->setGeometry(fieldX.toInt(&b), fieldY.toInt(&b), fieldWidth.toInt(&b), fieldHeight.toInt(&b));
			if(!widget->setOptions(fieldExtra))
				return 6;
			widget->show();
			}
		else if(fieldType=="label")
			{
			FeLabel *widget = new FeLabel(frame);
			widget->setName(fieldName);
			widget->setGeometry(fieldX.toInt(&b), fieldY.toInt(&b), fieldWidth.toInt(&b), fieldHeight.toInt(&b));
			if(!widget->setOptions(fieldExtra))
				return 7;
			widget->show();
			}
		else if(fieldType=="checkbox")
			{
			FeCheckBox *widget = new FeCheckBox(frame);
			widget->setName(fieldName);
			widget->setGeometry(fieldX.toInt(&b), fieldY.toInt(&b), fieldWidth.toInt(&b), fieldHeight.toInt(&b));
			if(!widget->setOptions(fieldExtra))
				return 12;
			widget->show();
			}
		else if(fieldType=="date")
			{
			FeDate *widget = new FeDate(frame);
			widget->setName(fieldName);
			widget->setGeometry(fieldX.toInt(&b), fieldY.toInt(&b), fieldWidth.toInt(&b), fieldHeight.toInt(&b));
			if(!widget->setOptions(fieldExtra))
				return 98;
			widget->show();
			}
		else
			return 8;
		}		
	return 0;
	}	
		
int FormEditorView::valid(void)
	{
	if(frame->title().isEmpty())
		return -1;
	return 0;
	}	
		
	
int FormEditorView::save(ISqlServer *sql)
	{
	int r = valid();
	if(r != 0)
		return r;
	if(!sql)
		return 1;
	// Create entry in forms tabls
	QString request(64000);
	request.sprintf("INSERT INTO __KmysqlForms (Name, Query, Help, Width, Height) VALUES ('%s', '%s', '%s', %d, %d)",
									(const char *)Utils::insertQuotes(frame->title()), (const char *)Utils::insertQuotes(frame->query()), (const char *)Utils::insertQuotes(frame->help()), frame->width(), frame->height());
	//warning("%s", (const char *)request); --> CRASHES IF request > 512 bytes !!!!!!!!
	QueryResult res = sql->query(request);
	if(res.type == QueryResult::Error)
		return 2;
	// Get the key correspondig to the form
	request = QString("SELECT Ref FROM __KmysqlForms WHERE Name='")+frame->title()+"'";
	//warning("%s", (const char *)request); --> CRASHES IF request > 512 bytes !!!!!!!!
	res = sql->query(request);
	if(res.type == QueryResult::Error)
		return 3;
	QString id = res.result.at(0);
	// Create entries for each item
	const QObjectList *children = frame->children();
	if(!children)
		return 0;
  QObjectListIt it(*children);
  QObject *obj;
	while ((obj=it.current()) != 0)
		{
		++it;
		QString className = obj->className();
		if(className == "FeButton")
			{
			if(!((FeButton *)obj)->writeDatabase(sql, id))
				return 5;
			}
		else if(className == "FeCombo")
			{
			if(!((FeCombo *)obj)->writeDatabase(sql, id))
				return 6;
			}
		else if(className == "FeEdit")
			{
			if(!((FeEdit *)obj)->writeDatabase(sql, id))
				return 7;	
			}
		else if(className == "FeLabel")
			{
			if(!((FeLabel *)obj)->writeDatabase(sql, id))
				return 8;			
			}
		else if(className == "FeCheckBox")
			{
			if(!((FeCheckBox *)obj)->writeDatabase(sql, id))
				return 9;			
			}
		else if(className == "FeDate")
			{
			if(!((FeDate *)obj)->writeDatabase(sql, id))
				return 10;			
			}
		else
			return 4;
		}
	return 0;
	}


void FormEditorView::setEnabled(bool enabled)
	{
	if(enabled)
		{
		show();
		menu->show();
		mainWindow()->setMenu(menu);
		mainWindow()->enableToolBar(KToolBar::Show, tbIdMain);
		mainWindow()->enableToolBar(KToolBar::Show, tbIdItems);
		mainWindow()->setView(this);
		}
	else
		{
		hide();
		menu->hide();
		mainWindow()->enableToolBar(KToolBar::Hide, tbIdMain);
		mainWindow()->enableToolBar(KToolBar::Hide, tbIdItems);
		}
	}




















































