/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mainview.h"
#include "connectionmanager.h"
#include "mainwindow.h"
#include "DialogInput.h"
#include "formdialog.h"
#include "utils.h"
#include "dlgpassword.h"

MainView::MainView(MainWindow *parent)
  : View(parent)
  {
  pluginManager = PluginManager::getInstance();
  connectionManager = ConnectionManager::getInstance();
  setFrameStyle(NoFrame);
  initWidgets();
  initToolBars();
  initMenus();
  initConnects();
  loadGeometry();
  readPref();
  if(pref.autoDetectPlugins)
  	pluginManager->autodetectPlugins();
  lastProductiveQuery = "";
  updateTree();
  };

void MainView::initConnects(void)
	{
	connect(btn_go, SIGNAL(clicked()), SLOT(go()));
  connect(edit, SIGNAL(submit()), SLOT(go()));
  connect(history, SIGNAL(selected(int)), SLOT(historyActivated(int)));
  connect(history, SIGNAL(edit(int)), SLOT(historyEdit(int)));
	connect(history, SIGNAL(execute(int)), SLOT(historyExecute(int)));
  connect(treeView, SIGNAL(tableSelect()), SLOT(slotTableSelect()));
  connect(treeView, SIGNAL(tableSelectAll()), SLOT(slotTableSelectAll()));
  connect(treeView, SIGNAL(tableShowFields()), SLOT(slotShowFields()));
  connect(treeView, SIGNAL(tableDrop()), SLOT(slotDropTable()));
  connect(treeView, SIGNAL(tableEdit()), SLOT(slotTableEdit()));
	connect(treeView, SIGNAL(tableRename()), SLOT(slotTableRename()));
	connect(treeView, SIGNAL(tableAddColumnFirst()), SLOT(slotTableAddColumnFirst()));
	connect(treeView, SIGNAL(tableAddColumnLast()), SLOT(slotTableAddColumnLast()));
  connect(treeView, SIGNAL(formExecute()), SLOT(slotFormExecute()));
  connect(treeView, SIGNAL(formEdit()), SLOT(slotFormEdit()));
  connect(treeView, SIGNAL(formDelete()), SLOT(slotFormDelete()));
  connect(treeView, SIGNAL(baseCreateTable()), SLOT(slotCreateTable()));
  connect(treeView, SIGNAL(baseDelete()), SLOT(slotDeleteBase()));
  connect(treeView, SIGNAL(baseSelect()), SLOT(slotSelect()));
  connect(treeView, SIGNAL(baseNewForm()), SLOT(slotBaseNewForm()));
	connect(treeView, SIGNAL(baseNewStaticQuery()), SLOT(slotBaseNewStaticQuery()));
	connect(treeView, SIGNAL(basePurgeForms()), SLOT(slotBasePurgeForms()));
  connect(treeView, SIGNAL(serverConnect()), SLOT(slotConnect()));
  connect(treeView, SIGNAL(serverDisconnect()), SLOT(slotClose()));
	connect(treeView, SIGNAL(serverCreateBase()), SLOT(slotCreateBase()));
  connect(treeView, SIGNAL(serverRemove()), SLOT(slotRemoveServer()));
  connect(treeView, SIGNAL(serverShutdown()), SLOT(slotShutdownServer()));
  connect(treeView, SIGNAL(serverReload()), SLOT(slotReloadServer()));
  connect(treeView, SIGNAL(serverInfo()), SLOT(slotServerInfo()));
  connect(treeView, SIGNAL(queryEdit()), SLOT(slotStaticQueryEdit()));
  connect(treeView, SIGNAL(queryExecute()), SLOT(slotStaticQueryExecute()));
  connect(treeView, SIGNAL(queryDelete()), SLOT(slotStaticQueryDelete()));
	connect(treeView, SIGNAL(columnShow()), SLOT(slotColumnShow()));
	connect(treeView, SIGNAL(columnEdit()), SLOT(slotColumnEdit()));
	connect(treeView, SIGNAL(columnDrop()), SLOT(slotColumnDrop()));	
	connect(treeView, SIGNAL(columnAdd()), SLOT(slotColumnAdd()));	
  //connect(treeView, SIGNAL(queryWizard()), SLOT(slotQueryWizard()));
  connect(mainWindow(), SIGNAL(quit()), this, SLOT(slotQuit()));
  connect(viewsEdit, SIGNAL(returnPressed()), this, SLOT(slotViewNew2()));
	}

void MainView::initToolBars(void)
	{
	KIconLoader *iconLoader = KApplication::getKApplication()->getIconLoader();
	QFrame *separatorLine;
	// TB1
	KToolBar *tb1 = new KToolBar(mainWindow());			
	//  ------
	tb1->insertButton(iconLoader->loadIcon("cserver.xpm"), 1, SIGNAL(clicked()), this, SLOT(slotAddServer()), true, i18n("Add server"));
	tb1->insertButton(iconLoader->loadIcon("plugin.xpm"), 2, SIGNAL(clicked()), this, SLOT(slotModules()), true, i18n("Plugins"));
	tb1->insertButton(iconLoader->loadIcon("configure.xpm"), 3, SIGNAL(clicked()), this, SLOT(slotPref()), true, i18n("Settings..."));
	//	------
	separatorLine = new QFrame(tb1);
  separatorLine->setFrameStyle(QFrame::VLine|QFrame::Sunken);
  tb1->insertWidget(0, 20, separatorLine);
  //  ------
	tb1->insertWidget(0, 120, (viewsEdit = new QLineEdit(tb1)));
	tb1->insertButton(iconLoader->loadIcon("reload.xpm"), 1, SIGNAL(clicked()), this, SLOT(slotClear()), true, i18n("Clear view"));
	tb1->insertButton(iconLoader->loadIcon("delete.xpm"), 2, SIGNAL(clicked()), this, SLOT(slotViewRemove()), true, i18n("Remove view"));
	tb1->insertButton(iconLoader->loadIcon("windowize.xpm"), 3, SIGNAL(clicked()), this, SLOT(slotViewWindowize()), true, i18n("Windowize"));
	//  ------
	separatorLine = new QFrame(tb1);
  separatorLine->setFrameStyle(QFrame::VLine|QFrame::Sunken);
  tb1->insertWidget(0, 20, separatorLine);
  //  ------
	tb1->insertButton(iconLoader->loadIcon("html.xpm"), 4, SIGNAL(clicked()), this, SLOT(slotViewHtmlExport()), true, i18n("Export HTML"));
	tb1->insertButton(iconLoader->loadIcon("fileprint.xpm"), 5, SIGNAL(clicked()), this, SLOT(slotPrint()), true, i18n("Print view"));	
	//  ------
	separatorLine = new QFrame(tb1);
  separatorLine->setFrameStyle(QFrame::VLine|QFrame::Sunken);
  tb1->insertWidget(0, 20, separatorLine);	
  //  ------
	tb1->insertButton(iconLoader->loadIcon("undo.xpm"), 3, SIGNAL(clicked()), edit, SLOT(undo()), true, i18n("Undo"));
	tb1->insertButton(iconLoader->loadIcon("redo.xpm"), 4, SIGNAL(clicked()), edit, SLOT(redo()), true, i18n("Redo"));
	tb1->insertSeparator();
	tb1->insertButton(iconLoader->loadIcon("editcut.xpm"), 5, SIGNAL(clicked()), edit, SLOT(cut()), true, i18n("Cut"));
	tb1->insertButton(iconLoader->loadIcon("editcopy.xpm"), 6, SIGNAL(clicked()), edit, SLOT(copy()), true, i18n("Copy"));
	tb1->insertButton(iconLoader->loadIcon("editpaste.xpm"), 7, SIGNAL(clicked()), edit, SLOT(paste()), true, i18n("Paste"));
	//  ------
	separatorLine = new QFrame(tb1);
  separatorLine->setFrameStyle(QFrame::VLine|QFrame::Sunken);
  tb1->insertWidget(0, 20, separatorLine);
	//  ------
  tb1->insertButton(iconLoader->loadIcon("exit.xpm"), 8, SIGNAL(clicked()), mainWindow(), SLOT(slotQuit()), true, i18n("Quit"));
  //  ------
  tbId1 = mainWindow()->addToolBar(tb1);
	}

void MainView::initMenus(void)
	{
	KIconLoader *iconLoader = KApplication::getKApplication()->getIconLoader();
	// Fichier
	QPopupMenu *fichier = new QPopupMenu();
  fichier->insertItem(iconLoader->loadIcon("cserver.xpm"), i18n("&Add server"), this, SLOT(slotAddServer()), CTRL+Key_S);
  fichier->insertItem(iconLoader->loadIcon("plugin.xpm"), i18n("&Plugins..."), this, SLOT(slotModules()));
  fichier->insertSeparator();
  fichier->insertItem(iconLoader->loadIcon("exit.xpm"), i18n("&Quit"), mainWindow(), SLOT(slotQuit()), CTRL+Key_Q);
  // Edition
  QPopupMenu *edition = new QPopupMenu();
  edition->insertItem(iconLoader->loadIcon("undo.xpm"), i18n("U&ndo"), edit, SLOT(undo()), CTRL+Key_Z);
  edition->insertItem(iconLoader->loadIcon("redo.xpm"), i18n("R&edo"), edit, SLOT(redo()), CTRL+Key_Y);
  edition->insertSeparator();
  edition->insertItem(iconLoader->loadIcon("editcut.xpm"), i18n("C&ut"), edit, SLOT(cut()), CTRL+Key_X);
  edition->insertItem(iconLoader->loadIcon("editcopy.xpm"), i18n("&Copy"), edit, SLOT(copy()), CTRL+Key_C);
  edition->insertItem(iconLoader->loadIcon("editpaste.xpm"), i18n("&Paste"), edit, SLOT(paste()), CTRL+Key_V);
  edition->insertSeparator();
  edition->insertItem(/*iconLoader->loadIcon(".xpm"), */i18n("I&ndent"), edit, SLOT(indent()), CTRL+Key_I);
  edition->insertItem(/*iconLoader->loadIcon(".xpm"), */i18n("&Unindent"), edit, SLOT(unIndent()), CTRL+Key_U);
  edition->insertSeparator();
  edition->insertItem(/*iconLoader->loadIcon(".xpm"), */i18n("Select &all"), edit, SLOT(selectAll()), CTRL+Key_A);
  edition->insertItem(/*iconLoader->loadIcon(".xpm"), */i18n("Deselect all"), edit, SLOT(deselectAll()));
  edition->insertItem(/*iconLoader->loadIcon(".xpm"), */i18n("Invert selection"), edit, SLOT(invertSelection()));
  // Views
  QPopupMenu *views = new QPopupMenu();
	views->insertItem(iconLoader->loadIcon("reload.xpm"), i18n("&Clear"), this, SLOT(slotClear()));
  views->insertItem(i18n("&Fit columns"), this, SLOT(slotFitAll()));
	views->insertSeparator();
  views->insertItem(iconLoader->loadIcon("filenew.xpm"), i18n("&New view"), this, SLOT(slotViewNew()), CTRL+Key_N);
  views->insertItem(iconLoader->loadIcon("delete.xpm"), i18n("&Remove"), this, SLOT(slotViewRemove()), CTRL+Key_D);
  views->insertItem(iconLoader->loadIcon("windowize.xpm"), i18n("&Window-ize"), this, SLOT(slotViewWindowize()), CTRL+Key_W);
  views->insertSeparator();
  views->insertItem(iconLoader->loadIcon("html.xpm"), i18n("&HTML export"), this, SLOT(slotViewHtmlExport()));
  views->insertItem(iconLoader->loadIcon("fileprint.xpm"), i18n("&Print..."), this, SLOT(slotPrint()), CTRL+Key_P);
  // Options
  QPopupMenu *options = new QPopupMenu();
  options->insertItem(iconLoader->loadIcon("configure.xpm"), i18n("S&ettings..."), this, SLOT(slotPref()));
  options->insertSeparator();
  options->insertItem(i18n("Editor &options..."), edit, SLOT(optDlg()));
	options->insertItem(i18n("Editor &colors..."), edit, SLOT(colDlg()));
	options->insertItem(i18n("Editor &defaults..."), edit, SLOT(hlDef()));
	options->insertItem(i18n("Syntax highlighting..."), edit, SLOT(hlDlg()));
  options->insertSeparator();
  options->insertItem(i18n("&Save settings"), this, SLOT(slotSavePref()));
  // Menu help
  QString about = i18n("This is KMySql version ");
  about += QString(VERSION) + "\n";
 	about += i18n("An SQL client for KDE.");
	about += "\n\nBy Frdrik Bilhaut\nkmysql@penguinpowered.com\nhttp://www.xnot.com/kmysql";
  QPopupMenu *help = KApplication::getKApplication()->getHelpMenu(true, about);
  // Menubar
  menu = new KMenuBar(mainWindow());
  menu->insertItem(i18n("&Server"), fichier);
  menu->insertItem(i18n("&Edit"), edition);
  menu->insertItem(i18n("&View"), views);
	menu->insertItem(i18n("&Options"), options);
  menu->insertSeparator();
  menu->insertItem(i18n("&Help"), help);
	}

void MainView::initWidgets(void)
	{
	// Le panner principal
  mainPanner = new KNewPanner(this, "mpanner", KNewPanner::Horizontal, KNewPanner::Percent, 70);
  QWidget *mainPannerChild0 = new QWidget(mainPanner);
  QWidget *mainPannerChild1 = new QWidget(mainPanner);
  mainPanner->activate(mainPannerChild0, mainPannerChild1);
  mainPanner->showLabels(true);
  QBoxLayout *topLayout = new QBoxLayout(mainPannerChild0, QBoxLayout::TopToBottom);
  QBoxLayout *botLayout = new QBoxLayout(mainPannerChild1, QBoxLayout::TopToBottom);
  // L'arbre et la vue
  panner = new KNewPanner(mainPannerChild0, "panner", KNewPanner::Vertical, KNewPanner::Percent, 30);
  QWidget *pannerChild0 = new QWidget(panner);
  QWidget *pannerChild1 = new QWidget(panner);
  panner->activate(pannerChild0, pannerChild1);
  QBoxLayout *leftLayout = new QBoxLayout(pannerChild0, QBoxLayout::LeftToRight);
  QBoxLayout *rightLayout = new QBoxLayout(pannerChild1, QBoxLayout::LeftToRight);
  treeView = new ServerTree(pannerChild0);
  leftLayout->addWidget(treeView);
  views = new TabViewGroup(pannerChild1);
  rightLayout->addWidget(views);
  topLayout->addWidget(panner, 100);
  // Query edit combo
  edit = new QueryEdit(mainPannerChild1);
  botLayout->addWidget(edit);
  // Le layout vertical
  QBoxLayout *vbox = new QBoxLayout(this, QBoxLayout::TopToBottom);
  vbox->addWidget(mainPanner, 100);
  vbox->addSpacing(5);
  // Le layout (pour les boutons)
  QBoxLayout *hbox = new QHBoxLayout;
  vbox->addLayout(hbox, 0);
	// HistoryButton
  history = new History(this);
  history->setFixedHeight(50);
  hbox->addWidget(history);
  hbox->addSpacing(5);
  // Go button
  btn_go = new QPushButton(i18n("Submit"), this);
  btn_go->setFixedSize(50, 50);
  hbox->addWidget(btn_go);
	}


MainView::~MainView(void)
	{
	}

void MainView::setEnabled(bool enabled)
	{
	if(enabled)
		{
		show();
		menu->show();
		mainWindow()->setMenu(menu);
		mainWindow()->enableToolBar(KToolBar::Show, tbId1);	
		mainWindow()->setView(this);
		}
	else
		{
		hide();
		menu->hide();
		mainWindow()->enableToolBar(KToolBar::Hide, tbId1);
		}
	}

	
int MainView::addConnection(Connection *conn)
  {
	int r = connectionManager->addConnection(conn);
	if(r == 0)
	  updateTree();
	return r;
  }

int MainView::removeConnection(Connection *conn)
	{
	int r = connectionManager->removeConnection(conn);
	if(r == 0)
		updateTree();
	return r;
	}
	
void MainView::slotConnect(void)
  {
  Connection *conn = currentConnection();
  if(!conn)
  	{
  	QMessageBox::warning(this, i18n("Connect"), i18n("Please select a server"));
  	return;
  	}
  QueryResult res;
  if(conn->getPassword().isEmpty())
  	{
  	DlgPassword dlg(this);
  	if(dlg.exec())
  		res = conn->connect(dlg.getPassword());
  	else
  		return;
  	}
  else
  	{
  	res = conn->connect();
  	}
  if(res.type == QueryResult::Error)
  	{
  	QMessageBox::warning(this, i18n("Connect error"), res.message);
  	return;
  	}
  updateTree();
  }


void MainView::slotClose(void)
  {
  ISqlServer *sql = currentServer();
  if(!sql)
  	{
  	QMessageBox::warning(this, i18n("Disconnect"), i18n("Please select a server"));
  	return;
  	}
  sql->close();
  updateTree();
  }

void MainView::slotPref(void)
  {
  DialogPref *dlg = new DialogPref(this);
  dlg->setFields(pref);
  if(dlg->exec())
    pref = dlg->getResult();
  delete dlg;
  }

void MainView::slotModules(void)
	{
	DialogModules *dlg = new DialogModules(pluginManager, this);
	if(dlg->exec())
		{
		}
	delete dlg;	
	}

void MainView::slotSavePref(void)
  {
  savePref();
  }

void MainView::slotPrint(void)
  {
  views->currentView()->print();
  }

void MainView::slotClear(void)
  {
  views->currentView()->clear();
  }


void MainView::slotFitAll(void)
  {
  views->currentView()->fitAll();
  }


void MainView::slotCreateTable(void)
  {
  QString base = currentBase();
  if(base.isEmpty())
    {
    QMessageBox::warning(this, i18n("Create table"), i18n("Please select a base"));
    return;
    }
  DialogCreateTable dlg(this);
  if(dlg.exec())
    {
    QString query = dlg.getQuery();
    warning(query);
    newQuery(query);
    }
  updateTree();
  }


void MainView::slotAlterTable(void)
  {
  updateTree();
  }


void MainView::slotDropTable(void)
  {
  QString table = currentTable();
  if(table.isEmpty())
    {
    QMessageBox::warning(this, i18n("Drop table"), i18n("Please select a table"));
    return;
    }
  QString tmp = i18n("Are you sure you want to drop this table: ");
  tmp += table;
  tmp += " ?";
  if(KMsgBox::yesNo(this, i18n("Drop Table"), (const char *)tmp, KMsgBox::QUESTION|KMsgBox::DB_SECOND) == 1)
    {
    QString query = "DROP TABLE "+table;
    newQuery(query);
    }
  updateTree();
  }

int MainView::newQuery(const char *query, bool showErrors, bool autoUpdate)
  {
  if(currentBase().isEmpty())
    {
    QMessageBox::warning(this, i18n("Query"), i18n("Please select a base"));
    return 0;
    }
  QueryResult res;
  if(!currentServer()->isConnected())
    {
    QMessageBox::critical(this, i18n("Error"), i18n("Not connected"));
    return 0;
    }
  res = currentServer()->selectBase(currentBase());
  if(res.type == QueryResult::Error)
    {
    QMessageBox::critical(this, i18n("Error"), res.message);
    return 0;
    }
  res = currentServer()->query(query);
  if(res.type == QueryResult::Error)
    {
    if(showErrors)
    	{
    	if(pref.errorsDestination == Pref::StatusBar)
    		mainWindow()->statusMessage(res.message);
    	else if(pref.errorsDestination == Pref::MessageBox)
    		QMessageBox::critical(this, i18n("Error"), res.message);
    	else
    		warning("Unknown destination for error");
    	}
    else
			views->currentView()->clear();
    return 0;
    }
  else if(res.type == QueryResult::Ok)
    {
    if(!lastProductiveQuery.isEmpty() && pref.autoUpdates && autoUpdate)
    	newQuery(lastProductiveQuery, false);
    mainWindow()->statusMessage(res.message);
    updateTree();
    return 1;
    }
  else if(res.type == QueryResult::Set)
    {
    lastProductiveQuery = query;
    views->currentView()->clear();
    views->currentView()->setAutoUpdate(false);
    int nbcols = res.numColumns;
    views->currentView()->setNumCols(nbcols);
    views->currentView()->setColumns(res.fields);
    QStrList rows = res.result;
    for(unsigned int i=0; i<rows.count(); i++)
      views->currentView()->appendItem(rows.at(i));
    mainWindow()->statusMessage(res.message, 0);
    views->currentView()->fitAll();
    views->currentView()->update();
    views->currentView()->setAutoUpdate(true);
    views->currentView()->repaint();
    updateTree();
    return 1;
    }
  return 0;
  }

void MainView::go(void)
  {
	if(newQuery(edit->text()))
  	{
  	edit->setFocus();
  	edit->selectAll();
  	addHistory(edit->text());
  	}
  }

void MainView::historyActivated(int i)
	{
	if(pref.historyBehaviour == Pref::Submit)
		historyExecute(i);
	else if(pref.historyBehaviour == Pref::Edit)
		historyEdit(i);
	else
		warning("Unknown history behaviour in pref.historyBehaviour");		
	}
	
void MainView::historyEdit(int i)
	{
	edit->setText(history->text(i));
	}
	
void MainView::historyExecute(int i)
	{
	newQuery(history->text(i));	
	}

int MainView::updateTree(Connection *conn)
	{
	if(!conn)
		return 1;
	// Si pas connect
	if(!conn->sql()->isConnected())
		{
		KTreeListItem *item = treeView->createServerItem(conn->getName());
		treeView->insertItem(item);
		return 0;
		}
	KTreeListItem *item = treeView->createConnectedServerItem(conn->getName());
  // On rcupre les diffrentes bases
  QueryResult res = conn->sql()->getBases();
  if(res.type == QueryResult::Error)
  	return 2;
  QStrList bases = res.result;
  for(unsigned int i=0; i<bases.count(); i++)
    {
    char *baseName = bases.at(i);
    // On rcupre les diffrentes tables
    QueryResult res = conn->sql()->getTables(baseName);
    if(res.type == QueryResult::Error  || res.type == QueryResult::Unsuported)
    	continue;
   	KTreeListItem *base = treeView->createBaseItem(baseName);
    QStrList tables = res.result;
    for(unsigned int j=0; j<tables.count(); j++)
 	    {
  	  QString tableName = tables.at(j);
   	  // Une table normale
   	  if(tableName != "__KmysqlForms" && tableName != "__KmysqlFields")
  		  {
   	  	KTreeListItem *table = treeView->createTableItem(tableName);
   	  	QueryResult res = conn->sql()->getColumns(baseName, tableName);
   	  	if(!(res.type == QueryResult::Error || res.type == QueryResult::Unsuported))
   	  		{
   	  		QStrList columns = res.result;
   	  		for(unsigned int k=0; k<columns.count(); k++)
   	  			{
   	  			QString columnName = columns.at(k);
   	  			KTreeListItem *column = treeView->createColumnItem(columnName);
   	  			table->appendChild(column);
   	  			}
   	  		}			
    		base->appendChild(table);
    		}
     	// Les forms
     	else if(tableName == "__KmysqlForms")
     		{
     		QueryResult res = conn->sql()->query("SELECT Name,Width,Height from __KmysqlForms");
     		if(res.type == QueryResult::Error)
   				continue;   				
   			QStrList forms = res.result;
   			for(unsigned int k=0; k<forms.count(); k++)
   				{
   				QString formName = Utils::subStringAt(forms.at(k), '\t',  0);
   				QString formWidth = Utils::subStringAt(forms.at(k), '\t', 1);
   				QString formHeight = Utils::subStringAt(forms.at(k), '\t', 2);
   				KTreeListItem *form;
   				if(formHeight.toInt() == -1 && formWidth.toInt() == -1)
   					form = treeView->createQueryItem(formName);
   				else
   					form = treeView->createFormItem(formName);
   				base->insertChild(0, form);
   				}
   			}
      }
    item->appendChild(base);
    }
  treeView->insertItem(item);
  return 0;
	}

int MainView::updateTree(void)
	{
	treeView->saveTree();
	treeView->setAutoUpdate(false);
	treeView->clear();
	for(unsigned int c=0; c<connectionManager->count(); c++)
		updateTree(connectionManager->at(c));
	treeView->repaint();
	treeView->setAutoUpdate(true);
	treeView->restoreTree();
	return 0;
	}


ISqlServer *MainView::currentServer(void)
  {
  Connection *conn = currentConnection();
  if(!conn)
  	return NULL;
  return conn->sql();
  }

Connection *MainView::currentConnection(void)
	{
	QString connName = treeView->currentConnectionName();
  if(connName.isEmpty())
    return NULL;
  Connection *conn = ConnectionManager::getInstance()->connectionByName(connName);
  return conn;
	}

QString MainView::currentBase(void)
  {
  return treeView->currentBaseName();
  }

QString MainView::currentTable(void)
  {
  return treeView->currentTableName();
  }

QString MainView::currentForm(void)
  {
  return treeView->currentFormName();
  }

void MainView::slotTableSelectAll(void)
	{
	QString query = "SELECT * FROM "+currentTable();
	newQuery(query);
	}

void MainView::slotTableSelect()
	{
	DialogSelect dlg(this, currentTable());
	if(dlg.exec())
		newQuery(dlg.getQuery());
	}
	
void MainView::slotTableEdit()
	{
	ISqlServer *sql = currentServer();
	sql->selectBase(currentBase());
	mainWindow()->teView()->edit(sql, treeView->currentTableName());
	mainWindow()->setMode(MainWindow::ModeTableEditor);
	}
	
void MainView::slotTableEditDone()
	{
	mainWindow()->setMode(MainWindow::ModeNormal);
	}
	
void MainView::slotTableRename()
	{
	ISqlServer *sql = currentServer();
	sql->selectBase(currentBase());
	QString tableName = treeView->currentTableName();
	DialogInput dlg(this);
	dlg.setMessage(i18n("New name:"));
	QString caption = i18n("Renaming table") + QString(" ") + tableName;
	dlg.setCaption(caption);
	if(dlg.exec())
		{
		QString newTableName = dlg.getText();
		QString query(1024);
		query.sprintf("ALTER TABLE %s RENAME AS %s", (const char *)tableName, (const char *)newTableName);
		newQuery(query);
		}
	}
	
void MainView::slotTableAddColumnFirst()
	{
	addColumn("FIRST");
	}

void MainView::slotTableAddColumnLast()
	{
	addColumn("");
	}	

	
void MainView::addColumn(QString position)
	{
	ISqlServer *sql = currentServer();
	sql->selectBase(currentBase());
	QString tableName = treeView->currentTableName();
	DialogTableField dlg(this);
	dlg.setPrimaryKeyEnabled(false);
	if(dlg.exec())
		{
		TableField *field = dlg.result();
		QString query(1024);
		query.sprintf("ALTER TABLE %s ADD COLUMN ", (const char *)tableName);
		query += field->toSQL() + " ";
		query += position;
		warning("%s", (const char *)query);
		newQuery(query);
		delete field;
		}
	}

void MainView::slotSelect(void)
	{
	DialogSelect dlg(this);
	if(dlg.exec())
		newQuery(dlg.getQuery());
	}
	
	
void MainView::slotShowFields()
	{
	QString table = currentTable();
	if(table.isEmpty())
		{
		QMessageBox::warning(this, i18n("Show fields"), i18n("Please select a table"));
		return;
		}
	newQuery("SHOW FIELDS FROM "+table);
	}
	
void MainView::slotDeleteBase()
	{
	QString baseName = currentBase();
	if(baseName.isEmpty())
		return;
	QString tmp = i18n("Are you sure tyou want to drop this base? ");
  tmp += baseName;
  tmp += " ?";
  if(KMsgBox::yesNo(this, i18n("Drop Base"), (const char *)tmp, KMsgBox::QUESTION|KMsgBox::DB_SECOND) == 1)
    {
    ISqlServer *sql = currentServer();
		if(!sql || !sql->isConnected())
			{
			QMessageBox::critical(this, i18n("Create base"), i18n("This shouldn't happen!"));
			return;
			}
    QueryResult res = sql->dropBase(baseName);
		if(res.type == QueryResult::Error)
			{
			QMessageBox::critical(this, i18n("Drop base"), res.message);
			return;
			}
		mainWindow()->statusMessage(i18n("Base dropped successfully"), 0);
		updateTree();
    }
	}
	
void MainView::slotCreateBase()
	{
	DialogInput dlg(this);
	dlg.setMessage(i18n("Base name:"));
	dlg.setCaption(i18n("Create base"));
	if(dlg.exec())
		{
		QString baseName = dlg.getText();
		ISqlServer *sql = currentServer();
		if(!sql || !sql->isConnected())
			{
			QMessageBox::critical(this, i18n("Create base"), i18n("This shouldn't happen!"));
			return;
			}
		QueryResult res = sql->createBase(baseName);
		if(res.type == QueryResult::Error)
			{
			QMessageBox::critical(this, i18n("Create base"), res.message);
			return;
			}
		mainWindow()->statusMessage(i18n("New base created successfully"), 0);
		updateTree();
		}
	}

void MainView::slotAddServer()
	{
	if(pluginManager->count() == 0)
		{
		QMessageBox::critical(this, i18n("Add server"), i18n("You must install at least one plugin"));
		return;
		}	
	DlgServer dlg(pluginManager, this);
	if(dlg.exec())
		{
		// Alloue une nouvelle connection.
		// Elle sera dsaloue automatiquement par la QList connections
		Connection *conn = dlg.result();
		int r = addConnection(conn);
		if(r == 1)
			{
			delete conn;
			QMessageBox::critical(this, i18n("Add server"), i18n("Such a connection already exists"));
			}
		}
	}
	
void MainView::slotRemoveServer()
	{
	removeConnection(currentConnection());
	}
	
void MainView::slotShutdownServer()
	{
	Connection *conn = currentConnection();
	if(!conn)
		return;
	ISqlServer *sql = conn->sql();
	if(!sql)
		return;
	if(!sql->isConnected())
		return;
	QueryResult res = sql->shutdown();
	if(res.type == QueryResult::Error)
		{
		QMessageBox::critical(this, i18n("Shutdown"), res.message);
		return;
		}
	removeConnection(conn);
	mainWindow()->statusMessage(i18n("Server stoped successfully"), 0);
	}

void MainView::slotReloadServer()
	{
	Connection *conn = currentConnection();
	if(!conn)
		return;
	ISqlServer *sql = conn->sql();
	if(!sql)
		return;
	if(!sql->isConnected())
		return;
	QueryResult res = sql->reload();
	if(res.type == QueryResult::Error)
		{
		QMessageBox::critical(this, i18n("Reload"), res.message);
		return;
		}
	updateTree();	
	mainWindow()->statusMessage(i18n("Server reloaded successfully"), 0);
	}

void MainView::slotServerInfo(void)
	{
	ISqlServer *sql = currentServer();
	if(sql && sql->isConnected())
		QMessageBox::information(this, i18n("Server information"), sql->serverInfo());
	}


void MainView::addHistory(QString query)
	{
	query = query.stripWhiteSpace();
	for(unsigned int i=0; i<history->count(); i++)
		{
		QString s = history->text(i);
		if(s == query)
			return;
		}
	history->insertItem(query, 0);
	if(history->count() > pref.nbsaved)
		history->removeItem(history->count()-1);
	}
	
void MainView::slotBaseNewForm(void)
	{
	QueryResult res;
	ISqlServer *sql = currentServer();
	if(!sql)
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in menuBaseNewForm()");
		return;
		}
	QString baseName = currentBase();
	sql->selectBase(baseName);
	if(!FormUtils::isBasePreparedForForms(sql, baseName))
		if(!FormUtils::prepareBaseForForms(sql, baseName))
			{
			QMessageBox::critical(this, i18n("New form"), i18n("Unable to create forms in this base (check permissions)"));
			return;
			}
	if(!FormUtils::isBaseInCurrentFormat(sql, baseName))
		if(!FormUtils::updateBaseToCurrentFormat(sql, baseName))
			return;
	mainWindow()->feView()->clear();
	feMode = FeModeNew;
	feBaseName = baseName;
	mainWindow()->statusMessage(i18n("New form"));
	mainWindow()->setMode(MainWindow::ModeFormEditor);
	}


void MainView::slotFormExecute(void)
	{
	ISqlServer *sql = currentServer();
	if(!sql)
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in menuFormExecute():1");
		return;
		}
	QString baseName = currentBase();
	if(!FormUtils::isBaseInCurrentFormat(sql, baseName))
		if(!FormUtils::updateBaseToCurrentFormat(sql, baseName))
			return;
	sql->selectBase(baseName);
	QString formName = currentForm();
	if(formName.isEmpty())
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in menuFormExecute():2");
		return;
		}		
	FormDialog dlg(this);
	int r = dlg.load(sql, formName);
	if(r != 0)
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in menuFormExecute():4");
		return;
		}
	if(dlg.exec())
		{
		QString queries = dlg.makeQuery();
		QStrList queryList;
		Utils::splitQueries(queries, queryList);
		for(unsigned int i=0; i<queryList.count(); i++)
			{
			QString query = queryList.at(i);
			warning("Form query no %d: %s", i, (const char *)query);
			newQuery(query, true, (i == queryList.count()-1));
			}
		}
	}
	
void MainView::slotFormEdit(void)
	{
	ISqlServer *sql = currentServer();
	if(!sql)
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in menuFormEdit():1");
		return;
		}
	QString baseName = currentBase();
	sql->selectBase(baseName);
	QString formName = currentForm();
	if(formName.isEmpty())
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in menuFormEdit():2");
		return;
		}
	if(!FormUtils::isBaseInCurrentFormat(sql, baseName))
		if(!FormUtils::updateBaseToCurrentFormat(sql, baseName))
			return;
	int r = mainWindow()->feView()->load(sql, formName);
	if(r != 0)
	  {
		QString msg;
		msg.sprintf("Internal error number %d while loading form", r);
		QMessageBox::critical(this, i18n("Critical"), msg);
		return;
		}
	oldFormName = formName;
	feMode = FeModeEdit;
	feBaseName = baseName;
	mainWindow()->statusMessage(i18n("Editing form ")+oldFormName);
	mainWindow()->setMode(MainWindow::ModeFormEditor);
	}
	
void MainView::slotFormDelete(void)
	{
	ISqlServer *sql = currentServer();
	if(!sql)
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in menuFormDelete():1");
		return;
		}
	QString baseName = currentBase();
	sql->selectBase(baseName);
	QString formName = currentForm();
	if(formName.isEmpty())
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in menuFormDelete():2");
		return;
		}			
	if(!FormUtils::removeFormFromDatabase(sql, formName))
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in menuFormDelete():3");
		return;
		}
	updateTree();
	}


void MainView::slotFormSave(void)
	{
	ISqlServer *sql = currentServer();
	if(!sql)
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in menuFormSave():1");
		return;
		}
	sql->selectBase(feBaseName);
	if(feMode == FeModeNew)
		{
		int r = mainWindow()->feView()->save(sql);
		if(r > 0)
			{
			QString msg;
			msg.sprintf("Internal error number %d while saving form", r);
			QMessageBox::critical(this, i18n("Critical"), msg);
			return;
			}
		else if(r < 0)
			{
			switch(r)
				{
				case -1:
					QMessageBox::critical(this, i18n("Invalid form"), i18n("Please give a title to your form"));
					return;
				}
			}
		updateTree();
		mainWindow()->setMode(MainWindow::ModeNormal);
		}
	else if(feMode == FeModeEdit)
		{
		if(!FormUtils::removeFormFromDatabase(sql, oldFormName))
			{
			QMessageBox::critical(this, i18n("Critical"), "Internal error while deleting form");
			return;
			}
		int r = mainWindow()->feView()->save(sql);
		if(r > 0)
			{
			QString msg;
			msg.sprintf("Internal error number %d while saving form", r);
			QMessageBox::critical(this, i18n("Critical"), msg);
			return;
			}
		else if(r < 0)
			{
			switch(r)
				{
				case -1:
					QMessageBox::critical(this, i18n("Invalid form"), i18n("Please give a title to your form"));
					return;
				}
			}
		updateTree();
		mainWindow()->setMode(MainWindow::ModeNormal);
		}
	}
	
void MainView::slotFormCancel(void)
	{
	mainWindow()->setMode(MainWindow::ModeNormal);
	}

void MainView::slotBaseNewStaticQuery(void)
	{
	DlgStaticQuery dlg(this);
	if(dlg.exec())
		{
		QString queryName = dlg.getQueryName();
		QString queryText = dlg.getQueryText();
		QueryResult res;
		ISqlServer *sql = currentServer();
		if(!sql)
			{
			QMessageBox::critical(this, i18n("Critical"), "Internal error in menuBaseNewStaticQuery():1");
			return;
			}
		QString baseName = currentBase();
		sql->selectBase(baseName);
		if(!FormUtils::isBasePreparedForForms(sql, baseName))
			if(!FormUtils::prepareBaseForForms(sql, baseName))
				{
				QMessageBox::critical(this, i18n("New form"), i18n("Unable to create forms in this base (check permissions)"));
				return;
				}
		if(!FormUtils::isBaseInCurrentFormat(sql, baseName))
			if(!FormUtils::updateBaseToCurrentFormat(sql, baseName))
				return;
		QString query = "INSERT INTO __KmysqlForms (Name, Query, Help, Width, Height) VALUES ('" +
										Utils::insertQuotes(queryName) + "', '" + Utils::insertQuotes(queryText) + "', '', -1, -1)";
		res = sql->query(query);
		if(res.type == QueryResult::Error)
			{
			QMessageBox::critical(this, i18n("Critical"), "Internal error in menuBaseNewStaticQuery():2");
			return;
			}
		updateTree();
		}
	}
	
void MainView::slotBasePurgeForms()
	{
	QString message(512);
	message.sprintf(i18n("This will delete all forms stored in the base '%s', and related kmysql internal tables.\nThis may be useful if you encounter serious problems with your forms.\nDo you wish to continue?"), (const char *)currentBase());
	if(KMsgBox::yesNo(this, i18n("Purge forms"), (const char *)message, KMsgBox::QUESTION|KMsgBox::DB_SECOND) == 1)
		{
		QueryResult res;
		ISqlServer *sql = currentServer();	
		QString baseName = currentBase();
		sql->selectBase(baseName);
		res = sql->query("DROP TABLE __KmysqlForms");
		if(res.type != QueryResult::Ok)
			{
			QMessageBox::critical(this, i18n("Critical"), i18n("Failed to drop the __KmysqlForms table. Check permissions."));
			return;
			}
		res = sql->query("DROP TABLE __KmysqlFields");
		if(res.type != QueryResult::Ok)
			{
			QMessageBox::critical(this, i18n("Critical"), i18n("Failed to drop the __KmysqlFields table. Check permissions."));
			return;
			}
		updateTree();
		}
	}
	
void MainView::slotStaticQueryEdit(void)
	{
	QueryResult res;
	ISqlServer *sql = currentServer();
	if(!sql)
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in slotStaticQueryEdit():1");
		return;
		}
	QString baseName = currentBase();
	sql->selectBase(baseName);
	QString queryName = treeView->currentQueryName();
	res = sql->query("SELECT Ref, Query FROM __KmysqlForms WHERE Name='"+Utils::insertQuotes(queryName)+"'");
	if(res.type == QueryResult::Error)
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in slotStaticQueryExecute():2");
		return;
		}
	QString formRef = Utils::subStringAt(res.result.at(0), '\t', 0);
	QString queryText = Utils::subStringAt(res.result.at(0), '\t', 1);
	DlgStaticQuery dlg(this);
	dlg.setQueryText(queryText);
	dlg.setQueryName(queryName);
	if(dlg.exec())
		{
		queryName = dlg.getQueryName();
		queryText = dlg.getQueryText();
		res = sql->query("UPDATE __KmysqlForms SET Name='"+Utils::insertQuotes(queryName)+"', Query='"+Utils::insertQuotes(queryText)+"' WHERE Ref="+formRef);
		if(res.type == QueryResult::Error)
			{
			QMessageBox::critical(this, i18n("Critical"), "Internal error in slotStaticQueryExecute():3");
			return;
			}
		updateTree();
		}
	}
	
void MainView::slotStaticQueryExecute(void)
	{
	QueryResult res;
	ISqlServer *sql = currentServer();
	if(!sql)
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in slotStaticQueryExecute():1");
		return;
		}
	QString baseName = currentBase();
	sql->selectBase(baseName);
	QString queryName = treeView->currentQueryName();
	res = sql->query("SELECT Query FROM __KmysqlForms WHERE Name='"+Utils::insertQuotes(queryName)+"'");
	if(res.type == QueryResult::Error)
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in slotStaticQueryExecute():2");
		return;
		}
	QString queries = res.result.at(0);
	QStrList queryList;
	Utils::splitQueries(queries, queryList);
	for(unsigned int i=0; i<queryList.count(); i++)
		{
		QString query = queryList.at(i);
		warning("Static query no %d: %s", i, (const char *)query);
		newQuery(query, true, (i == queryList.count()-1));
		}
	}
	
void MainView::slotStaticQueryDelete(void)
	{
	QueryResult res;
	ISqlServer *sql = currentServer();
	if(!sql)
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in slotStaticQueryDelete():1");
		return;
		}
	QString baseName = currentBase();
	sql->selectBase(baseName);
	QString queryName = treeView->currentQueryName();
	res = sql->query("DELETE FROM __KmysqlForms WHERE Name='"+Utils::insertQuotes(queryName)+"'");
	if(res.type == QueryResult::Error)
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in slotStaticQueryDelete():2");
		return;
		}
	updateTree();
	}
	
void MainView::slotQuit()
	{
	saveGeometry();
	if(pref.savequit)
    savePref();
	}
		
int MainView::readPref(void)
  {
  KConfig *config = KApplication::getKApplication()->getConfig();
  // Read plugins config
  pluginManager->readPref(config);
  // Load KWrite config
  edit->readConfig(config);
  // Read all connections.
 	int count;
  config->setGroup("Connections");
  count = config->readNumEntry("Count", 0);
  for(int i=0; i < count; i++)
	  {
    QString host, user, passwd, pluginName, portStr;
    bool b;
    int port = 0;
	  QString nr;
	  QStrList list;
    nr.sprintf("%d", i);
    config->readListEntry(nr, list);
    host = list.first();
    user = list.next();
    passwd = list.next();
    pluginName = list.next();		
		portStr = list.next();
		if(!portStr.isEmpty())
			port = portStr.toInt(&b);
		int id = pluginManager->getPluginByName(pluginName);
		if(id == -1)
			QMessageBox::critical(this, i18n("Configuration"), i18n("A connection can't be restored because the plugin can't be found."));
		else
			{
			ISqlServer *sql = pluginManager->createInstance(id);
			const char *ppath = pluginManager->getPluginPath(id);
			Connection *conn = new Connection(sql, ppath, pluginName, host, user, passwd, port);
  	 	addConnection(conn);
  	 	}
 		}
 	// Other configs...
  config->setGroup("Options");
  pref.savequit = config->readBoolEntry("SaveQuit", false);
  pref.nbsaved = config->readNumEntry("MaxSaved", 10);
  pref.historyBehaviour = config->readNumEntry("HistoryBehaviour", 0);
  pref.errorsDestination = config->readNumEntry("ErrorsDestination", 1);
  pref.autoUpdates = config->readBoolEntry("AutoUpdates", true);
  pref.autoDetectPlugins = config->readBoolEntry("AutoDetectPlugins", true);
  return 1;
  }

int MainView::savePref(void)
  {
  KConfig *config = KApplication::getKApplication()->getConfig();
  // Save plugin config
  pluginManager->savePref(config);
  // Save KWrite config
  edit->writeConfig(config);
  // Save all connections
  int i = 0;
  config->setGroup("Connections");
  config->writeEntry("Count", connectionManager->count());
  for(Connection *aConnect = connectionManager->first(); aConnect != NULL; aConnect = connectionManager->next() )
  	{
  	QStrList list;
  	QString portStr;
  	portStr.setNum(aConnect->getPort());
	  QString nr;
 	 	nr.sprintf("%d", i);
    list.append(aConnect->getHost());
    list.append(aConnect->getLogin());
    list.append(aConnect->getPassword());
    list.append(aConnect->getPluginName());
    list.append(portStr);
    config->writeEntry(nr, list);
    i++;
	  }
	// Other little things
  config->setGroup("Options");
  config->writeEntry("SaveQuit", pref.savequit);
  config->writeEntry("MaxSaved", pref.nbsaved);
  config->writeEntry("HistoryBehaviour", pref.historyBehaviour);
  config->writeEntry("ErrorsDestination", pref.errorsDestination);
  config->writeEntry("AutoUpdates", pref.autoUpdates);
  config->writeEntry("AutoDetectPlugins", pref.autoDetectPlugins);
  return 1;
  }

void MainView::loadGeometry(void)
	{
	KConfig *config = KApplication::getKApplication()->getConfig();
	config->setGroup("Geometry");
	mainPanner->setSeparatorPos(config->readNumEntry("MainPanner1", 70));
	panner->setSeparatorPos(config->readNumEntry("MainPanner2", 30));
	}
	
void MainView::saveGeometry(void)
	{
	KConfig *config = KApplication::getKApplication()->getConfig();
	config->setGroup("Geometry");
	config->writeEntry("MainPanner1", mainPanner->separatorPos());
	config->writeEntry("MainPanner2", panner->separatorPos());
	}

void MainView::slotViewNew()
	{
	DialogInput dlg(this);
	dlg.setMessage("View name:");
	if(dlg.exec())
		{
		QString viewName = dlg.getText();
		/*if(views->exists(viewName))
			{
			QMessageBox::warning(0, i18n("New view"), i18n("View name already in use, please give it another one."));
			slotViewNew();
			return;
			}*/
		views->newView(viewName);
		}
	}
	
	
void MainView::slotViewRemove()
	{
	if(!views->deleteCurrentView())
		QMessageBox::warning(0, i18n("Remove view"), i18n("There must remain at least one view!"));
	}
	
void MainView::slotViewWindowize()
	{
	if(!views->windowizeCurrentView())
		QMessageBox::warning(0, i18n("Windowize view"), i18n("Cannot windowize this view."));	
	}
	
void MainView::slotViewHtmlExport()
	{
	QString fileName = KFileDialog::getSaveFileName();
	if(fileName.isEmpty())
		return;
	QFile file(fileName);
	if(!file.open(IO_Truncate | IO_WriteOnly))
		{
		QMessageBox::critical(0, i18n("HTML export"), i18n("Error opening output file"));
		return;
		}
	if(!views->currentView()->exportHTML(&file, true, false))
		{
		QMessageBox::critical(0, i18n("HTML export"), i18n("Error generating HTML code"));
		file.close();
		return;
		}
	file.flush();
	file.close();
	}

void MainView::slotViewNew2()
	{
	QString viewName = viewsEdit->text();
	if(viewName.isEmpty())
		return;
	/*
	if(views->exists(viewName))
		{
		QMessageBox::warning(0, i18n("New view"), i18n("View name already in use, please give another one."));
		return;
		}*/
	views->newView(viewName);
	viewsEdit->setText("");
	}
	

void MainView::slotColumnShow()
	{
	ISqlServer *sql = currentServer();
	if(!sql)
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in slotColumnShow():1");
		return;
		}
	sql->selectBase(treeView->currentBaseName());
	QString column = treeView->currentColumnName();
	QString table = treeView->currentTableName();
	newQuery("SHOW FIELDS FROM "+table+" LIKE '"+column+"'");
	}

void MainView::slotColumnEdit()
	{
	ISqlServer *sql = currentServer();
	if(!sql)
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in slotColumnEdit():1");
		return;
		}
	sql->selectBase(treeView->currentBaseName());
	QString column = treeView->currentColumnName();
	QString table = treeView->currentTableName();
	if(column.isEmpty())
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in slotColumnEdit():2");
		return;
		}
	QueryResult res = sql->query("SHOW FIELDS FROM "+table);
	if(res.type != QueryResult::Set)
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in slotColumnEdit():3");
		return;
		}
	TableField *field = 0;
	for(unsigned int i=0; i<res.result.count(); i++)
		{
		QString row = res.result.at(i);
		QString colName = Utils::subStringAt(row, '\t', 0);
		if(colName != column)
			continue;		
		field = new TableField;
		field->name = column;
		QString totalType = Utils::subStringAt(row, '\t', 1);
		QString complexType = Utils::subStringAt(totalType, ' ', 0);
		char type[128];
		int length, decimals;
		int r = sscanf(complexType, "%[^(](%d,%d)", type, &length, &decimals);
		if(r == 1)
			{
			field->type = QString(type).upper();
			field->specLength = false;
			field->length = 1;
			field->specDecimals = false;
			field->decimals = 1;
			field->isUnsigned = false;
			}
		else if(r == 2)
			{
			field->type = QString(type).upper();
			field->specLength = true;
			field->length = length;
			field->specDecimals = false;
			field->decimals = 1;
			field->isUnsigned = false;
			}
		else if(r == 3)
			{
			field->type = QString(type).upper();
			field->specLength = true;
			field->length = length;
			field->specDecimals = true;
			field->decimals = decimals;
			field->isUnsigned = false;
			}	
		else
			{
			QMessageBox::critical(this, i18n("Critical"), "Internal error in slotColumnEdit():4");
			return;
			}
		field->isUnsigned = false;
		field->binary = false;
		field->zeroFill = false;
		for(int i=1; i<10; i++)
			{
			QString typeParam = Utils::subStringAt(totalType, ' ', i);
			if(typeParam.isEmpty())
				break;
			else if(typeParam == "unsigned")
				field->isUnsigned = true;
			else if(typeParam == "zerofill")
				field->zeroFill = true;
			else if(typeParam == "binary")
				field->binary = true;
			}
		QString canNull = Utils::subStringAt(row, '\t', 2);
		field->canNull = (canNull == "YES");
		QString key = Utils::subStringAt(row, '\t', 3);
		field->isKey = (key == "PRI");
		QString defaultValue = Utils::subStringAt(row, '\t', 4);
		field->specDefaultValue = !defaultValue.isEmpty();
		field->defaultValue = defaultValue;
		QString extra = Utils::subStringAt(row, '\t', 5);
		for(int i=0; i<10; i++)
			{
			QString param = Utils::subStringAt(extra, ' ', i);
			if(param.isEmpty())
				break;
			else if(param == "auto_increment")
				field->autoIncrement = true;
			}
		break;
		}
	DialogTableField dlg(this);
	dlg.initFields(field);
	dlg.setPrimaryKeyEnabled(false);
	if(dlg.exec())
		{
		TableField *newField = dlg.result();
		QString change = newField->toSQL();
		QString query = "ALTER TABLE "+table+" CHANGE COLUMN "+column+" "+change;
		warning("%s", (const char *)query);
		newQuery(query);
		delete newField;
		}
	delete field;
	}

void MainView::slotColumnDrop()
	{
	ISqlServer *sql = currentServer();
	if(!sql)
		{
		QMessageBox::critical(this, i18n("Critical"), "Internal error in slotColumnDrop():1");
		return;
		}
	sql->selectBase(treeView->currentBaseName());
	QString column = treeView->currentColumnName();
	QString table = treeView->currentTableName();
	if(column.isEmpty())
		return;
	QString tmp = i18n("Are you sure you want to drop this column: ");
  tmp += column;
  tmp += " ?";
  if(KMsgBox::yesNo(this, i18n("Drop column"), (const char *)tmp, KMsgBox::QUESTION|KMsgBox::DB_SECOND) != 1) // CANCEL
  	return;
  QString query(1024);
  query.sprintf("ALTER TABLE %s DROP COLUMN %s", (const char *)table, (const char *)column);
	QueryResult res = sql->query(query);
	if(res.type == QueryResult::Error)
		{
		QMessageBox::warning(this, i18n("Error droping column"), res.message);
		return;
		}	
  updateTree();
	}

void MainView::slotColumnAdd()
	{
	addColumn("AFTER "+treeView->currentColumnName());
	}





















































