/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MAINVIEW_H
#define MAINVIEW_H

#include <qlayout.h>
#include <qpushbutton.h>
#include <qprinter.h>
#include <qfile.h>

#include <knewpanner.h>
#include <kcombo.h>
#include <kapp.h>
#include <kmsgbox.h>
#include <kmenubar.h>
#include <kfiledialog.h>

#include "DialogCreateTable.h"
#include "queryedit.h"
#include "servertree.h"
#include "view.h"
#include "dlgserver.h"
#include "DialogSelect.h"
#include "connection.h"
#include "pluginmanager.h"
#include "DialogModules.h"
#include "formutils.h"
#include "pluginmanager.h"
#include "connectionmanager.h"
#include "DialogPref.h"
#include "tabviewgroup.h"
#include "dlgstaticquery.h"
#include "history.h"

class MainView : public View
  {
  Q_OBJECT
  public:
    MainView(MainWindow *);
    ~MainView(void);
    void setEnabled(bool);
  private:
  	void initWidgets(void);
  	void initToolBars(void);
  	void initMenus(void);
  	void initConnects(void);
  	int readPref(void);
    int savePref(void);
    int newQuery(const char *, bool = true, bool = true);
    int updateTree(void);
    int updateTree(Connection *);
    void addHistory(QString);
    ISqlServer *currentServer(void);
    Connection *currentConnection(void);
    Connection *connectionByName(QString);
    QString currentBase(void);
    QString currentTable(void);
    QString currentForm(void);
    int addConnection(Connection *);
    int removeConnection(Connection *);
    void loadGeometry(void);
    void saveGeometry(void);
    void addColumn(QString);
  private slots:
  	void slotConnect();
    void slotClose();
    void slotPref();
    void slotModules();
    void slotSavePref();
    void slotPrint();
    void slotClear();
    void slotFitAll();
    void slotCreateTable();
    void slotAlterTable();
    void slotDropTable();
    void slotTableSelect();
    void slotTableSelectAll();
    void slotTableEdit();
    void slotTableEditDone();
    void slotTableRename();
    void slotTableAddColumnFirst();
    void slotTableAddColumnLast();
    void slotBaseNewForm();
    void slotBaseNewStaticQuery();
    void slotShowFields();
    void slotDeleteBase();
    void slotCreateBase();
    void slotBasePurgeForms();
    void slotRemoveServer();
    void slotAddServer();
    void slotShutdownServer();
    void slotReloadServer();
    void slotServerInfo();
    void slotSelect();
    void slotFormExecute();
    void slotFormEdit();
    void slotFormDelete();
    void slotFormSave();
    void slotFormCancel();
    void slotStaticQueryExecute();
    void slotStaticQueryEdit();
    void slotStaticQueryDelete();
    void slotColumnShow();
    void slotColumnEdit();
    void slotColumnDrop();
    void slotColumnAdd();
    void historyActivated(int);
    void historyEdit(int);
    void historyExecute(int);
    void go();
    void slotQuit();
    void slotViewNew();
    void slotViewRemove();
    void slotViewWindowize();
    void slotViewHtmlExport();
    void slotViewNew2();
  private:
	  enum { FeModeNew, FeModeEdit };
  	KMenuBar *menu;
  	int tbId1, tbId2;
    QueryEdit *edit;
    TabViewGroup *views;
    ServerTree *treeView;
    QPushButton *btn_go;
    History *history;
  	QString oldFormName;
  	int feMode;
  	QString feBaseName;
  	Pref pref;
  	PluginManager *pluginManager;
  	ConnectionManager *connectionManager;
  	KNewPanner *mainPanner, *panner;
  	QLineEdit *viewsEdit;
  	QString lastProductiveQuery;
  };

#endif




















































