/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mainwindow.h"


MainWindow::MainWindow(const char *name)
  : KTMainWindow(name)
  {
  const int MAIN_MIN_W = 250;
  const int MAIN_MIN_H = 250;
  setMinimumSize(MAIN_MIN_W, MAIN_MIN_H);
  setCaption("KMySql " VERSION);
  status = statusBar();
  QString greeting = i18n("Welcome to");
  greeting += " KMySql " VERSION;
  status->insertItem(greeting , 0);
  _mainView = new MainView(this);
  _feView = new FormEditorView(this);
  _teView = new TableEditorView(this);
  loadGeometry();
  setMode(ModeNormal);
  }


void MainWindow::setMode(Mode mode)
	{
	_mainView->setEnabled(mode == ModeNormal);
	_feView->setEnabled(mode == ModeFormEditor);
	_teView->setEnabled(mode == ModeTableEditor);
	updateRects();
	}


void MainWindow::statusMessage(QString s, int pos)
	{
	status->changeItem(s, pos);	
	}

MainWindow::~MainWindow(void)
  {
  }

void MainWindow::closeEvent(QCloseEvent *e)
  {
  slotQuit();
  }

void MainWindow::slotQuit(void)
  {
  emit quit();
	saveGeometry();
  qApp->quit();
  }

void MainWindow::saveGeometry(void)
	{
	KConfig *config = KApplication::getKApplication()->getConfig();
	config->setGroup("Geometry");
	config->writeEntry("MainX", x());
	config->writeEntry("MainY", y());
	config->writeEntry("MainW", width());
	config->writeEntry("MainH", height());
	}
	
void MainWindow::loadGeometry(void)
	{
	const int DEFAULT_MAIN_W = 650;
	const int DEFAULT_MAIN_H = 550;
	KConfig *config = KApplication::getKApplication()->getConfig();
	config->setGroup("Geometry");
	int _x = config->readNumEntry("MainX", x());
	int _y = config->readNumEntry("MainY", y());
	int _w = config->readNumEntry("MainW", DEFAULT_MAIN_W);
	int _h = config->readNumEntry("MainH", DEFAULT_MAIN_H);
	setGeometry(_x, _y, _w, _h);
	}



		

























































