/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PLUGIN_H
#define PLUGIN_H

#include <dlfcn.h>
#include "iobject.h"
#include <qstring.h>

typedef IObject *(*CreateInstanceFuncType)(void);
typedef int (*DeleteInstanceFuncType)(IObject *);
typedef const char *(*GetNameFuncType)(void);
typedef const char *(*GetInfoFuncType)(void);

class Plugin
  {
  public:
    bool load(const char *libName);
    bool unload(void);
    IObject *createInstance();
    bool deleteInstance(IObject *);
    const char *getPluginName(void);
    const char *getPluginInfo(void);
    const char *getPluginPath(void);
    int getErrorCode(void);
    QString getErrorString(void);
  private:
    void *libPtr;
    QString errorString;
    int errorCode;
    QString path;
    CreateInstanceFuncType createFunc;
    DeleteInstanceFuncType deleteFunc;
    GetNameFuncType getNameFunc;
    GetInfoFuncType getInfoFunc;
  };

#endif







