/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "servertree.h"

ServerTree::ServerTree(QWidget *parent)
  : KTreeList(parent)
  {
  int id;
  // Les icones
  KIconLoader *iconLoader = KApplication::getKApplication()->getIconLoader();
  serverPix = iconLoader->loadIcon(QString("server.xpm"));
  connectedServerPix = iconLoader->loadIcon(QString("cserver.xpm"));
  basePix = iconLoader->loadIcon(QString("base.xpm"));
  tablePix = iconLoader->loadIcon(QString("table.xpm"));
  formPix = iconLoader->loadIcon(QString("form.xpm"));
  queryPix = iconLoader->loadIcon(QString("query.xpm"));
  columnPix = iconLoader->loadIcon(QString("column.xpm"));
  // Les popups
  QPopupMenu *admin = new QPopupMenu;
  admin->insertItem(i18n("Shutdown"), this, SLOT(slotServerShutdown()));
  admin->insertItem(i18n("Reload"), this, SLOT(slotServerReload()));
  admin->insertItem(i18n("Informations"), this, SLOT(slotServerInfo()));
  serverPopup = new QPopupMenu;
  serverPopup->insertItem(i18n("Connect"), this, SLOT(slotServerConnect()));
  serverPopup->insertItem(i18n("Remove"), this, SLOT(slotServerRemove()));
  connectedServerPopup = new QPopupMenu;
  connectedServerPopup->insertItem(i18n("Disconnect"), this, SLOT(slotServerDisconnect()));
  connectedServerPopup->insertItem(i18n("Remove"), this, SLOT(slotServerRemove()));
  connectedServerPopup->insertItem(i18n("Create base"), this, SLOT(slotServerCreateBase()));
  connectedServerPopup->insertItem(i18n("Administrate"), admin);
  basePopup = new QPopupMenu;
  basePopup->insertItem(i18n("Create table"), this, SLOT(slotBaseCreateTable()));
  basePopup->insertItem(i18n("Select..."), this, SLOT(slotBaseSelect()));
  basePopup->insertItem(i18n("New form"), this, SLOT(slotBaseNewForm()));
  basePopup->insertItem(i18n("New static query"), this, SLOT(slotBaseNewStaticQuery()));
  id = basePopup->insertItem(i18n("Query wizard..."), this, SLOT(slotQueryWizard()));
  basePopup->setItemEnabled(id, false);
  basePopup->insertSeparator();
  basePopup->insertItem(i18n("Drop base"), this, SLOT(slotBaseDelete()));
	basePopup->insertItem(i18n("Purge forms"), this, SLOT(slotBasePurgeForms()));
  tablePopup = new QPopupMenu;
  tablePopup->insertItem(i18n("Show fields"), this, SLOT(slotTableShowFields()));
  tablePopup->insertItem(i18n("Select in this table..."), this, SLOT(slotTableSelect()));
  tablePopup->insertItem(i18n("Select all in this table"), this, SLOT(slotTableSelectAll()));
  tablePopup->insertSeparator();
  tablePopup->insertItem(i18n("Edit contents"), this, SLOT(slotTableEdit()));
	tablePopup->insertSeparator();
  tablePopup->insertItem(i18n("Rename"), this, SLOT(slotTableRename()));
  tablePopup->insertItem(i18n("Add column first"), this, SLOT(slotTableAddColumnFirst()));
  tablePopup->insertItem(i18n("Add column last"), this, SLOT(slotTableAddColumnLast()));
  tablePopup->insertSeparator();
  tablePopup->insertItem(i18n("Drop table"), this, SLOT(slotTableDrop()));
  formPopup = new QPopupMenu;
  formPopup->insertItem(i18n("Execute"), this, SLOT(slotFormExecute()));
  formPopup->insertItem(i18n("Edit"), this, SLOT(slotFormEdit()));
	formPopup->insertSeparator();
  formPopup->insertItem(i18n("Delete"), this, SLOT(slotFormDelete()));
  queryPopup = new QPopupMenu;
  queryPopup->insertItem(i18n("Execute"), this, SLOT(slotQueryExecute()));
  queryPopup->insertItem(i18n("Edit"), this, SLOT(slotQueryEdit()));
	queryPopup->insertSeparator();
  queryPopup->insertItem(i18n("Delete"), this, SLOT(slotQueryDelete()));
  columnPopup = new QPopupMenu;
	columnPopup->insertItem(i18n("Show column"), this, SLOT(slotColumnShow()));
	columnPopup->insertItem(i18n("Add column after"), this, SLOT(slotColumnAdd()));
  columnPopup->insertItem(i18n("Edit column"), this, SLOT(slotColumnEdit()));
	columnPopup->insertSeparator();
	columnPopup->insertItem(i18n("Drop column"), this, SLOT(slotColumnDrop()));
  }

KTreeListItem *ServerTree::createConnectedServerItem(const char *name)
  {
  return new KTreeListItem(name, &connectedServerPix);
  }


KTreeListItem *ServerTree::createServerItem(const char *name)
  {
  return new KTreeListItem(name, &serverPix);
  }

KTreeListItem *ServerTree::createBaseItem(const char *name)
  {
  return new KTreeListItem(name, &basePix);
  }

KTreeListItem *ServerTree::createTableItem(const char *name)
  {
  return new KTreeListItem(name, &tablePix);
  }

KTreeListItem *ServerTree::createFormItem(const char *name)
	{
	return new KTreeListItem(name, &formPix);
	}
	
KTreeListItem *ServerTree::createQueryItem(const char *name)
	{
	return new KTreeListItem(name, &queryPix);
	}
	
KTreeListItem *ServerTree::createColumnItem(const char *name)
	{
	return new KTreeListItem(name, &columnPix);
	}

ServerTree::ItemType ServerTree::currentItemType(void)
  {
  KPath *path = itemPath(currentItem());
  if(path == NULL)
    return UNDEF;
  int count = path->count();
  delete path;
  switch(count)
    {
    case 1:
      {
      KTreeListItem *item = itemAt(currentItem());
      if(item->getPixmap()->serialNumber() == connectedServerPix.serialNumber())
      	return CSERVER;
     	else
				return DSERVER;
      }
    case 2:
    	{
    	return BASE;
    	}
    case 3:
    	{
    	KTreeListItem *item = itemAt(currentItem());
     	if(item->getPixmap()->serialNumber() == formPix.serialNumber())
    		return FORM;
     	else if(item->getPixmap()->serialNumber() == tablePix.serialNumber())
	    	return TABLE;
	    else if(item->getPixmap()->serialNumber() == queryPix.serialNumber())
	    	return QUERY;
	    else
	    	return UNDEF;
    	}
    case 4:
    	{
    	return COLUMN;
    	}
    default:
    	return UNDEF;
    }
  return UNDEF;
  }


QString ServerTree::currentConnectionName(void)
  {
  int i = currentItem();
  if(i == -1)
    return 0;
  KPath *path = itemPath(i);
  QString *s = NULL;
  while(!path->isEmpty())
    {
    if(s)
      delete s;
    s = path->pop();
    }
  if(s==NULL)
    return 0;
  QString res = *s;
  delete path;
  return res;
  }

QString ServerTree::currentBaseName(void)
  {
  int i = currentItem();
  if(i==-1)
    return 0;
  KPath *path = itemPath(i);
  if(!path)
    return 0;
  if(path->count() < 2)
    return 0;
  while(path->count() != 2)
    delete(path->pop());
  QString *s = path->pop();
  QString res = *s;
  delete s;
  delete path;
  return res;
  }

QString ServerTree::currentTableName(void)
  {
  int i = currentItem();
  if(i==-1)
    return 0;
  KPath *path = itemPath(i);
  if(!path)
    return 0;
  if(path->count() < 3)
    return 0;
  while(path->count() != 3)
    delete(path->pop());
  QString *s = path->pop();
  QString res = *s;
  delete s;
  delete path;
  return res;
  }

QString ServerTree::currentColumnName(void)
  {
  int i = currentItem();
  if(i==-1)
    return 0;
  KPath *path = itemPath(i);
  if(!path)
    return 0;
  if(path->count() < 4)
    return 0;
  while(path->count() != 4)
    delete(path->pop());
  QString *s = path->pop();
  QString res = *s;
  delete s;
  delete path;
  return res;
  }

QString ServerTree::currentFormName(void)
	{
	return currentTableName();
	}
	
QString ServerTree::currentQueryName(void)
	{
	return currentTableName();
	}

void ServerTree::mousePressEvent(QMouseEvent *e)
  {
  KTreeList::mousePressEvent(e);
  if(e->button()==RightButton)
    {
    QPopupMenu *popup = 0;
    switch(currentItemType())
      {
      case CSERVER:
      	popup = connectedServerPopup;
      	break;
      case DSERVER:
      	popup = serverPopup;
	      break;
      case BASE:
      	popup = basePopup;
      	break;
      case TABLE:
				popup = tablePopup;
				break;
			case FORM:
				popup = formPopup;
				break;
			case QUERY:
				popup = queryPopup;
				break;
			case COLUMN:
				popup = columnPopup;
				break;
      default:
				return;
      }
    if(popup)
    	popup->popup(mapToGlobal(e->pos()));
    }
  }

void ServerTree::mouseDoubleClickEvent(QMouseEvent *e)
  {
  KTreeList::mouseDoubleClickEvent(e);
	switch(currentItemType())
    {
    case CSERVER:
   		/* what to do here ? */
   		break;
		case DSERVER:
	 		emit serverConnect();      	
      break;
    case BASE:
    	/* what to do here ? */ ;
 	  	break;
    case TABLE:
			emit tableSelectAll();
			break;
		case FORM:
			emit formExecute();
			break;
		case QUERY:
			emit queryExecute();
			break;
		case COLUMN:
			emit columnShow();
			break;
   	default:
			return;
    }
  }

void ServerTree::slotServerConnect(void)
	{
	emit serverConnect();
	}

void ServerTree::slotServerCreateBase(void)
	{
	emit serverCreateBase();
	}

void ServerTree::slotServerDisconnect(void)
	{
	emit serverDisconnect();
	}

void ServerTree::slotServerRemove(void)
	{
	emit serverRemove();
	}

void ServerTree::slotServerShutdown(void)
	{
	emit serverShutdown();
	}

void ServerTree::slotServerReload(void)
	{
	emit serverReload();
	}

void ServerTree::slotServerInfo(void)
	{
	emit serverInfo();
	}

void ServerTree::slotBaseCreateTable(void)
	{
	emit baseCreateTable();
	}

void ServerTree::slotBaseDelete(void)
	{
	emit baseDelete();
	}
	
void ServerTree::slotBasePurgeForms(void)
	{
	emit basePurgeForms();
	}	
	
void ServerTree::slotBaseNewStaticQuery(void)
	{
	emit baseNewStaticQuery();
	}

void ServerTree::slotTableDrop(void)
	{
	emit tableDrop();
	}

void ServerTree::slotTableSelect(void)
	{
	emit tableSelect();
	}
	
void ServerTree::slotTableSelectAll(void)
	{
	emit tableSelectAll();
	}

void ServerTree::slotTableShowFields(void)
	{
	emit tableShowFields();
	}

void ServerTree::slotTableEdit(void)
	{
	emit tableEdit();
	}
	
void ServerTree::slotTableRename(void)
	{
	emit tableRename();
	}	
	
void ServerTree::slotTableAddColumnFirst(void)
	{
	emit tableAddColumnFirst();
	}	
	
void ServerTree::slotTableAddColumnLast(void)
	{
	emit tableAddColumnLast();
	}	

void ServerTree::slotColumnShow(void)
	{
	emit columnShow();
	}	
		
void ServerTree::slotColumnEdit(void)
	{
	emit columnEdit();
	}
	
void ServerTree::slotColumnAdd(void)
	{
	emit columnAdd();
	}
	
void ServerTree::slotColumnDrop(void)
	{
	emit columnDrop();
	}
	
void ServerTree::slotBaseSelect(void)
	{
	emit baseSelect();
	}
	
void ServerTree::slotBaseNewForm(void)
	{
	emit baseNewForm();
	}

void ServerTree::slotQueryWizard(void)
	{
	emit queryWizard();
	}

void ServerTree::slotFormExecute()
	{
	emit formExecute();
	}
	
void ServerTree::slotFormEdit()
	{
	emit formEdit();
	}

void ServerTree::slotFormDelete()
	{
	emit formDelete();
	}
	
void ServerTree::slotQueryExecute()
	{
	emit queryExecute();
	}
	
void ServerTree::slotQueryEdit()
	{
	emit queryEdit();
	}

void ServerTree::slotQueryDelete()
	{
	emit queryDelete();
	}
		
void ServerTree::saveTree(void)
	{
	selectedItem = currentItem();
	nodes.setAutoDelete(true);
	nodes.clear();
	forEveryItem(&_saveTree, (void *)this);
	}

void ServerTree::restoreTree(void)
	{
	forEveryItem(&_restoreTree, (void *)this);
	if(itemAt(selectedItem) != NULL)
		setCurrentItem(selectedItem);
	}

bool _saveTree(KTreeListItem *item, void *_vthis)
	{
	ServerTree *_this = (ServerTree *)_vthis;
	QString path = _this->getPath(item);
	ServerTree::NodeInfo *node = new ServerTree::NodeInfo(path, item->isExpanded());
	_this->nodes.append(node);
	return false;
	}

bool _restoreTree(KTreeListItem *item, void *_vthis)
	{
	ServerTree *_this = (ServerTree *)_vthis;
	QString path = _this->getPath(item);
	ServerTree::NodeInfo *node = _this->nodeInfoByPath(path);
	if(node != NULL)
		if(node->expanded)
			_this->expandItem(_this->itemIndex(item));
	return false;
	}

ServerTree::NodeInfo *ServerTree::nodeInfoByPath(QString path)
	{
	for(unsigned int i=0; i<nodes.count(); i++)
		{
		NodeInfo *node = nodes.at(i);
		if(node->path == path)
			return node;
		}
	return NULL;
	}

QString ServerTree::pathToString(KPath *path)
	{
	if(!path)
		return 0;
	QString res;
	while(!path->isEmpty())
		{
		QString *s = path->pop();
		res += (*s);
		res += "|";
		}
	delete path;
	return res;
	}

QString ServerTree::getPath(KTreeListItem *item)
	{
	QString res;
	if(item->hasParent())
		res = getPath(item->getParent());
	res += item->getText();
	res += "|";
	return res;
	}


// Nodeinfo
ServerTree::NodeInfo::NodeInfo(QString path, bool expanded)
	: path(path), expanded(expanded)
	{
	}

ServerTree::NodeInfo::~NodeInfo(void)
	{
	}














	













