/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SERVERTREE_H
#define SERVERTREE_H

#include <qpixmap.h>
#include <qpopupmenu.h>
#include <ktreelist.h>
#include <kapp.h>
#include <kiconloader.h>

class ServerTree : public KTreeList
  {
  Q_OBJECT
  friend bool _saveTree(KTreeListItem *, void *);
  friend bool _restoreTree(KTreeListItem *, void *);
  public:
    enum ItemType { CSERVER, DSERVER, BASE, TABLE, FORM, QUERY, COLUMN, UNDEF };
    ServerTree(QWidget *);
    KTreeListItem *createServerItem(const char *);
    KTreeListItem *createConnectedServerItem(const char *);
    KTreeListItem *createBaseItem(const char *);
    KTreeListItem *createTableItem(const char *);
    KTreeListItem *createFormItem(const char *);
    KTreeListItem *createQueryItem(const char *);
    KTreeListItem *createColumnItem(const char *);
    ItemType currentItemType(void);
    QString currentConnectionName(void);
    QString currentBaseName(void);
    QString currentTableName(void);
    QString currentFormName(void);
		QString currentQueryName(void);
		QString currentColumnName(void);
    void saveTree(void);
    void restoreTree(void);
  signals:
    void serverConnect();
    void serverDisconnect();
    void serverRemove();
    void serverShutdown();
    void serverReload();
    void serverInfo();
    void serverCreateBase();
    void baseCreateTable();
    void baseDelete();
    void baseSelect();
    void baseNewForm();
    void baseNewStaticQuery();
    void basePurgeForms();
    void tableDrop();
    void tableShowFields();
    void tableSelect();
    void tableSelectAll();
    void tableRename();
    void tableAddColumnFirst();
		void tableAddColumnLast();
    void tableEdit();
    void queryExecute();
    void queryEdit();
    void queryDelete();
    void formExecute();
    void formEdit();
    void formDelete();
    void columnShow();
    void columnEdit();
    void columnDrop();
    void columnAdd();
    void queryWizard();
  protected:
    void mousePressEvent(QMouseEvent *);
		void mouseDoubleClickEvent(QMouseEvent *);
  private slots:
		void slotServerConnect();
		void slotServerDisconnect();
    void slotServerRemove();
    void slotServerShutdown();
    void slotServerReload();
    void slotServerInfo();
    void slotServerCreateBase();
    void slotBaseCreateTable();
    void slotBaseDelete();
    void slotBaseSelect();
    void slotBaseNewForm();
    void slotBaseNewStaticQuery();
    void slotBasePurgeForms();
    void slotTableDrop();
    void slotTableShowFields();
    void slotTableSelect();
    void slotTableSelectAll();
    void slotTableEdit();
    void slotTableRename();
    void slotTableAddColumnFirst();
    void slotTableAddColumnLast();
    void slotQueryWizard();
    void slotFormExecute();
    void slotFormEdit();
    void slotFormDelete();
    void slotQueryEdit();
    void slotQueryDelete();
    void slotQueryExecute();
		void slotColumnShow();
    void slotColumnEdit();
    void slotColumnDrop();
    void slotColumnAdd();
  private:
  	class NodeInfo;
  	QString pathToString(KPath *);
  	QString getPath(KTreeListItem *);
  	NodeInfo *nodeInfoByPath(QString);
  private:
  	class NodeInfo
		  {
		  public:	
				NodeInfo(QString, bool);
				~NodeInfo(void);
				QString path;
				bool expanded;
			};	
    QPixmap connectedServerPix, serverPix, basePix, tablePix, formPix, queryPix, columnPix;
    QPopupMenu *serverPopup, *connectedServerPopup, *basePopup, *tablePopup, *formPopup, *queryPopup, *columnPopup;
    QList<NodeInfo> nodes;
    int selectedItem;
  };

#endif // SERVERTREE_H









