/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <htmlview.h>
#include <qmessagebox.h>
#include "tabview.h"
#include "utils.h"

TabView::TabView(QWidget *parent, const char *name)
	: EditableTabListBox(parent, name)
	{
	setSeparator('\t');
  setAutoUpdate(true);	
	}

void TabView::fitAll(void)
	{
	QFontMetrics metrics(tableFont());
	for(int i=0; i<numCols(); i++)
		{
		int max = metrics.width(getHeadText(i));
		for(int j=0; j<numRows(); j++)
			{
			QString s = text(j, i);
			int tmp = metrics.width(s, s.length());
			if(tmp>max)
				max = tmp;
			}
		setColumnWidth(i, max+10);
		}
	update();
	}


void TabView::setColumns(QString columns)
	{
	if(columns.isEmpty())
		return;
	heads.clear();
	int index = 0;
	bool fin = false;
	int i = 0;
	while(!fin)
		{
		int index2 = columns.find('\t', index);
		if(index2 == -1)
			{
			index2 = columns.length();
			fin = true;
			}
		QString col = columns.mid(index, index2-index);
		setColumn(i, col);
		heads.append(col);
		index = index2+1;
		i++;
		}
	}
	
QString TabView::getHeadText(unsigned int index)
	{
	if(index<0 || index>=heads.count())
		return 0;
	return heads.at(index);	
	}

bool TabView::exportHTML(QIODevice *IODevice, bool caption, bool embedable)
	{
	QTextStream out(IODevice);
	// En-tte
	if(!embedable)
		{
		out << "<HTML>" << endl;
		out << "<BODY>" << endl;
		}
	// Debut de la table
	out << "<TABLE BORDER>" << endl;
	// Le titre
	if(caption)
		out << "  <CAPTION ALIGN=\"TOP\">" << name() << "</CAPTION>" << endl;
	// Les titres de colonnes
	out << "  <TR>" << endl;
	for(int col=0; col<numCols(); col++)	
		out << "    <TH>" << getHeadText(col) << "</TH>" << endl;;
	out << "  </TR>" << endl;
	// Le tableau en lui-mme
	for(int row=0; row<numRows(); row++)
		{
		out << "  <TR>" << endl;
		for(int col=0; col<numCols(); col++)
			out << "    <TD>" << text(row, col) << "</TD>" << endl;
		out << "  </TR>" << endl;
		}
	// Fin de la table
	out << "</TABLE>" << endl;
	// Pied de page
	if(!embedable)
		{
		out << "</BODY>" << endl;
		out << "</HTML>" << endl;
		}
	return true;
	}	
	
	
void TabView::print(void)
	{
	QBuffer buffer;
	if(!buffer.open(IO_ReadWrite))
		{
		QMessageBox::critical(0, "Error", "Internal error in TabView::print():1");
		return;
		}
	if(!exportHTML(&buffer, true, false))
		{
		QMessageBox::critical(0, "Error", "Internal error in TabView::print():2");
		return;
		}
	KHTMLView *view = new KHTMLView;
	view->begin();
	view->write(buffer.buffer());
	view->end();
	view->parse();
	view->print();
	}	
	
	



















