/***************************************************************************
    copyright            : (C) 1999 by Frederik Bilhaut
    email                : bilhaut_f@mail.cpod.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "utils.h"

	
QString Utils::subStringAt(QString str, char separator, unsigned int index)
	{
	if(str.isEmpty())
		return "";
	int begin = 0;
	int end = 0;
	for(unsigned int i = 0; i<index; i++)
		{
		begin = str.find(separator, begin);
		if(begin == -1)
			return "";
		begin++;
		}
	end = str.find(separator, begin);
	if(end == -1)
		end = str.length();
	QString res = str.mid(begin, end-begin);	
	return res.stripWhiteSpace();
	}
	
QString Utils::insertQuotes(QString str)
	{
	QString res;
	for(unsigned int i=0; i<str.length(); i++)
		{
		if(str[i] == '\'')
			res += "\\'";
		else
			res += str[i];
		}
	return res;
	}


void Utils::splitQueries(QString queries, QStrList &result)
	{	
	enum State { Normal, SimpleQuotedString, DoubleQuotedString };
	QString current = "";
	unsigned int len = queries.length();
	State state = Normal;
	for(unsigned int i=0; i<len; i++)
		{
		char prev = i>0 ? queries[i-1] : '\0';
		char c = queries[i];
		switch(state)
			{
			case Normal:
				{
				if(c == '\'' && prev != '\\')
					state = SimpleQuotedString;
				else if(c == '"' && prev != '\\')
					state = DoubleQuotedString;
				if(c == ';')
					{
					if(!current.stripWhiteSpace().isEmpty())
						{
						result.append(current.stripWhiteSpace().copy());
						current = "";						
						}
					}		
				else
					current += c;								
				break;
				}
			case SimpleQuotedString:
				{
				if(c == '\'')
					state = Normal;
				current += c;
				break;
				}
			case DoubleQuotedString:
				{
				if(c == '"')
					state = Normal;
				current += c;
				break;
				}
			default:
				warning("Utils::splitQueries(): unknown state");
			}
		}
	if(!current.stripWhiteSpace().isEmpty())
		result.append(current.stripWhiteSpace().copy());
	}
	

void Utils::Option::clear(void)
	{
	params.clear();
	name = "";
	}
	

bool Utils::Option::fromString(QString str)
	{
	clear();
	int len = str.length();
	int pos = 0;
	QString s;
	while(pos < len && str[pos] != '(')
		{
		s += str[pos];
		pos++;
		}
	name = s;
	s = "";
	if(pos==len)
		return true;
	pos++;
	for(;;)
		{
		if(pos == len)
			return false;
		char c = str[pos];
		if(c == '\\')
			{
			pos++;
			char d = str[pos];
			if(d == ',')
				s += ',';
			else if(d=='(')
			  s += '(';
			else if(d==')')
			  s += ')';
			else if(d=='\\')
			  s += '\\';			
			else
				{
				s += c;
				s += d;
				}
			}
		else if(c == ',')
			{
			params.append(s);
			s = "";
			}
		else if(c == ')')
			{
			params.append(s);
			return true;
			}
		else
			s += c;
		pos++;
		}
	return true;
	}
	
	
QString Utils::Option::toString(void)
	{
	QString res = name;
	unsigned int count = getParamCount();
	if(count == 0)
		return res;
	res += '(';
	for(unsigned int i=0; i<count; i++)
		{
		res += insertEscapeChars(getParam(i));
		if(i!=count-1)
			res += ',';
		}
	res += ')';
	return res;
	}	
	
QString Utils::Option::insertEscapeChars(QString str)
	{
	QString res = "";
	for(unsigned int i=0; i<str.length(); i++)
		{
		char c = str[i];
		if(c == ',')
			res += "\\,";
		else if(c == '(')
			res += "\\(";
		else if(c == ')')
			res += "\\)";		
		else if(c == '\\')
			res += "\\\\";
		else
		  res += c;		
		}
	return res;
	}
	
	
QString Utils::Option::getName(void)
	{
	return name;
	}
	
unsigned int Utils::Option::getParamCount(void)
	{
	return params.count();
	}
	
QString Utils::Option::getParam(unsigned int index)
	{
	QString res = strdup(params.at(index));
	return res;
	}
	
void Utils::Option::setName(QString n)
	{
	name = n;
	}	
	
void Utils::Option::addParam(QString s)
	{
	params.append(insertEscapeChars(s));
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	




