/***************************************************************************
                         STabCtl part of KDEStudio
                             -------------------
    copyright            : (C) 1999 by Judin Maximus
    email                : novaprint@mtu-net.ru
 ***************************************************************************/

#include "stabbar.h"
#include "b_left.xpm"
#include "b_right.xpm"

#include <qapp.h>
#include <qpixmap.h>


BarPainter::BarPainter( STabBar* parent )
:QWidget( parent )
{
  buffer = new QPixmap(0,0);
}

BarPainter::~BarPainter()
{
  delete buffer;
}

void BarPainter::paintEvent( QPaintEvent* )
{
  if ( !buffer->isNull() )
   bitBlt( this, 0, 0, buffer, 0, 0, width(), height() );
}

void BarPainter::resizeEvent( QResizeEvent *e )
{
  delete buffer;
  buffer = new QPixmap( width(), height() );
  drawBuffer();
}

void BarPainter::drawBuffer()
{
  if ( buffer->isNull() ) return;

  QPainter paint;
  paint.begin(buffer);
  paint.fillRect( 0, 0, width(), height(), colorGroup().background() );

  int x = 2;
  int curTab  = ((STabBar*)parent())->_currentTab;
  int curTabNum = -1;
  int leftTab = ((STabBar*)parent())->leftTab;
  int curx = -1; // start current tab ( selected )
  int curWidth = -1;
  int broken = -1;
  bool iconShow = ((STabBar*)parent())->iconShow;

  QList<TabBarData> *mainData = ((STabBar*)parent())->mainData;
  for ( uint k = 0; k < mainData->count(); k++ ){
    int x1 = x;
    int y1 = 2;
    int width = mainData->at(k)->width;

    if ( mainData->at(k)->id == curTab ){  // store current tab start x
      curTabNum = k;
      curx = x;
      curWidth = width;
      x1 -= 1;
      y1 -= 1;
    }

    if ( mainData->at(k)->pix != 0L && iconShow )
      paint.drawPixmap( x1+ 11, y1 + 2 , *mainData->at(k)->pix );

    if ( mainData->at(k)->enabled )
      paint.setPen( black );
    else
      paint.setPen( colorGroup().dark() );

    int ty = ( height() + fontMetrics().height() ) / 2 - 2;
    if ( mainData->at(k)->pix != 0L && iconShow )
      paint.drawText( x1 + 32 , ty + y1 , mainData->at(k)->label );
    else
      paint.drawText( x1 + 12 , ty + y1 , mainData->at(k)->label );

    paint.setPen( white );
    paint.moveTo( x1, height() + 1 -y1 );
    paint.lineTo( x1, y1 );

    paint.lineTo( x1 + width - 1, y1 );

    paint.setPen( colorGroup().dark() );
    paint.lineTo( x1 + width - 1, height()+1-y1 );

/***************************************************************/
    paint.setPen( white );
    paint.moveTo( x1 + 4, y1 + height() - 5 );
    paint.lineTo( x1 + 4, 3+y1 );

    paint.moveTo( x1 + 7, y1 + height() - 5 );
    paint.lineTo( x1 + 7, 3+y1 );

    paint.setPen( colorGroup().dark() );
    paint.moveTo( x1 + 5, y1 + height() - 5 );
    paint.lineTo( x1 + 5, 3+y1 );

    paint.moveTo( x1 + 8, y1 + height() - 5 );
    paint.lineTo( x1 + 8, 3+y1 );
/***************************************************************/

    // fixed picture for leftTab
    if ( leftTab == (int)k + 1 ){
      paint.setPen( colorGroup().background() );
      paint.moveTo( x1 + width - 1, 0 );
      paint.lineTo( x1 + width - 1, height() -1 );
      paint.moveTo( x1 + width - 2, 0 );
      paint.lineTo( x1 + width - 2, height() -1 );
    }

    // paint broken left
    if ( (int)k == leftTab && k != 0 )
    {
      int yy = y1;
      int xx = x1 - 2;
      paint.fillRect( x1, 0, 1, height() - 1, colorGroup().background() );
      paint.setPen( white );
      do {
          paint.drawPoint( xx + 2, yy );
          paint.drawPoint( xx + 1, yy + 1 );
          paint.moveTo( xx + 1, yy + 1 );
          paint.lineTo( xx + 1, yy + 3 );
          paint.drawPoint( xx + 2, yy + 4 );
          paint.lineTo( xx + 2, yy + 6 );
          paint.drawPoint( xx + 3, yy + 7 );
          paint.lineTo( xx + 3, yy + 9 );
          paint.drawPoint( xx + 2, yy + 10 );
          paint.drawPoint( xx + 2, yy + 11 );
          yy+= 12;
      } while ( yy < height() );
    }

    x += width;
    if ( x >= QWidget::width() && broken == -1 ) broken = k; // store right broken tab
  }

  // modify ( paint ) selected tab
  if ( curx != -1 && curTabNum >= leftTab ){
    curx -= 2;
    curWidth += 4;
    paint.setPen( white );
    paint.moveTo( curx, height()-1 );
    paint.lineTo( curx, 0 );
    paint.lineTo( curx + curWidth - 2, 0 );

    paint.setPen( colorGroup().dark() );
    paint.moveTo( curx + curWidth - 1, 0 );
    paint.lineTo( curx + curWidth - 1, height()-1 );

    paint.fillRect( curx + 1, 1, 2, height() - 1, colorGroup().background() );
    paint.fillRect( curx + curWidth - 4, 1, 3, height() - 1, colorGroup().background() );
    paint.fillRect( curx + 1, 1, curWidth - 3, 2, colorGroup().background() );
  }

  //    
  if ( curTabNum == leftTab && curTabNum != 0 )
  {
    int yy = 0;
    int xx = curx;
    paint.fillRect( curx, 0, 1, height() - 1, colorGroup().background() );
    paint.setPen( white );
    do {
        paint.drawPoint( xx + 2, yy );
        paint.drawPoint( xx + 1, yy + 1 );
        paint.moveTo( xx + 1, yy + 1 );
        paint.lineTo( xx + 1, yy + 3 );
        paint.drawPoint( xx + 2, yy + 4 );
        paint.lineTo( xx + 2, yy + 6 );
        paint.drawPoint( xx + 3, yy + 7 );
        paint.lineTo( xx + 3, yy + 9 );
        paint.drawPoint( xx + 2, yy + 10 );
        paint.drawPoint( xx + 2, yy + 11 );
        yy+= 12;
    } while ( yy < height() );
  }

  // paint broken right
  if ( broken != -1 )
  {
    int yy = broken == curTabNum ? 0:2;
    int xx = width();
    paint.fillRect( xx - 2, 0, 2, height() - 1, colorGroup().background() );
    paint.fillRect( xx - 5, yy + 1, 5, height() - 2 - yy, colorGroup().background() );
    paint.setPen( colorGroup().dark() );
    do {
        paint.drawPoint( xx - 2, yy );
        paint.drawPoint( xx - 1, yy + 1 );
        paint.moveTo( xx - 1, yy + 1 );
        paint.lineTo( xx - 1, yy + 3 );
        paint.drawPoint( xx - 2, yy + 4 );
        paint.lineTo( xx - 2, yy + 6 );
        paint.drawPoint( xx - 3, yy + 7 );
        paint.lineTo( xx - 3, yy + 9 );
        paint.drawPoint( xx - 2, yy + 10 );
        paint.drawPoint( xx - 2, yy + 11 );
        yy+= 12;
    } while ( yy < height() );
  }

  bitBlt( this, 0, 0, buffer, 0, 0, width(), height() );
  paint.end();
}

int BarPainter::findBarByPosX( int x )
{
  int dx = 5; // overlaped

  STabBar* bar = (STabBar*)parent();

  QList<TabBarData> *mainData = bar->mainData;
  if ( mainData->isEmpty() ) return -1;

  int end = 0;
  int find = -1;
  int findWidth = -1;
  for ( uint k = 0; k < mainData->count(); k++ ){
    end += mainData->at(k)->width;
    if ( x < end ){
      find = k;
      findWidth = mainData->at(k)->width;
      break;
    }
  }

  int idCurTab = bar->_currentTab;
  int curTab = -1;
  for ( uint k = 0; k < mainData->count(); k++ )
    if ( mainData->at(k)->id == idCurTab ){
      curTab = k;
      break;
    }

  // prosecc first Tab manualy
  if ( x < dx && curTab != mainData->at(0)->id ) return -1;

  // prosecc last Tab manyaly
  if ( find == -1 )
    if ( x < (end + dx) && curTab == (int)mainData->count() - 1 )
      find = mainData->count() - 1;

  if ( find == -1 ) return -1;

  // process overlaped
  if ( find > 0 )
    if ( curTab == (find - 1) &&  x < (end - findWidth + dx ) ) find -= 1;

  if ( find < (int)mainData->count() - 1 )
    if ( curTab == (find + 1) &&  x > (end - dx ) ) find += 1;

  return mainData->at(find)->id;
}

void BarPainter::mousePressEvent( QMouseEvent* e )
{
  int cur = findBarByPosX( e->x() );
  if ( e->button() == RightButton )
    emit ((STabBar*)parent())->rightButtonPress( cur, e->globalPos() );
  else
    mousePressTab = cur;
}

void BarPainter::mouseReleaseEvent( QMouseEvent* e )
{
  int cur = findBarByPosX( e->x() );
  if ( cur != -1 && cur == mousePressTab ){
    ((STabBar*)parent())->setCurrentTab( cur );
  }
}

/***************************************************************************/

STabBar::STabBar( QWidget * parent, const char * name )
:QWidget( parent, name )
{
  iconShow = true;

  barPainter = new BarPainter( this );
  move( 0, 0 );

  mainData = new QList<TabBarData>;
  mainData->setAutoDelete( true );
  _currentTab = -1;
  leftTab = 0;

  left = new QPushButton(this); left->setPixmap( QPixmap(b_left_xpm) ); left->hide();
  connect( left, SIGNAL(clicked()), SLOT( leftClicked()) );
  right = new QPushButton(this); right->setPixmap( QPixmap(b_right_xpm) ); right->hide();
  connect( right, SIGNAL(clicked()), SLOT( rightClicked()) );

  setFixedHeight( fontMetrics().height() + 10 );
}

STabBar::~STabBar()
{
  delete barPainter;
  delete left;
  delete right;
  delete mainData;
}


void STabBar::paintEvent(QPaintEvent *)
{
  QPainter paint;
  paint.begin( this );

  // find current ( selected ) tab data
  TabBarData* data = 0L;
  int curx = 2 + barPainter->x();
  int curWidth = 0;
  for ( uint k = 0; k < mainData->count(); k++ ){
    data = mainData->at(k);
    if ( data->id == _currentTab ){
      curWidth = data->width + 4 ;
      curx -= 2;
      break;
    }
    curx += data->width;
  }

  if ( curWidth == 0 ) curx = 0; // no tab selected

  // paint white button line
  paint.setPen( white );
  paint.moveTo( 0, height()-1 );
  paint.lineTo( curx, height()-1 );

  paint.moveTo( QMIN( curx + curWidth, width() - 50 ), height()-1 );
  paint.lineTo( width() - 1, height()-1 );

  paint.end();
}

int STabBar::insertTab( const char* label, int id )
{
  if ( id == -1 ){
    id = 0;
    for ( uint k = 0; k < mainData->count(); k++ )
      if ( mainData->at(k)->id > id ) id = mainData->at(k)->id;
  }
  TabBarData* data = new TabBarData( id, label );

  data->width = 4 + fontMetrics().width( label ) + 14;
  mainData->append( data );

  QSize size( width(), height() );
  qApp->notify( this, new QResizeEvent( size , size ) );
  return id;
}

void STabBar::showTabIcon( bool show )
{
  if ( iconShow == show ) return;
  iconShow = show;
  for ( uint k = 0; k < mainData->count(); k++ ){
    if (  iconShow && mainData->at(k)->pix != 0L ) mainData->at(k)->width += 16 + 4;
    if ( !iconShow && mainData->at(k)->pix != 0L ) mainData->at(k)->width -= 16 + 4;
  }

  if ( iconShow )
    setFixedHeight( fontMetrics().height() + 10 );
  else
    setFixedHeight( fontMetrics().height() + 4 );

  QSize size( width(), height() );
  qApp->notify( this, new QResizeEvent( size , size ) );
  repaint();
}

TabBarData* STabBar::findData( int id )
{
  TabBarData* data = 0L;
  for ( uint k = 0; k < mainData->count(); k++ )
    if ( mainData->at(k)->id == id ){
      data = mainData->at(k);
      break;
    }

  return data;
}

void STabBar::setPixmap( int id, const QPixmap &pix )
{
  TabBarData* data = findData( id );
  if ( data != 0L ){
    data->pix = new QPixmap( pix );
    if ( iconShow ) data->width += 16 + 4;
    QSize size( width(), height() );
    qApp->notify( this, new QResizeEvent( size , size ) );
  }
}

void STabBar::removeTab( int id )
{
  TabBarData* data = findData( id );
  if ( data != 0L ){
    if ( _currentTab == data->id )
    {
      for ( uint k = 0; k < mainData->count(); k++ )
      {
        if ( mainData->at(k)->id == data->id ){
          if ( mainData->count() == 1 ){
            setCurrentTab( -1 );
          } else {
            setCurrentTab( mainData->at(k+1)->id );
          }
          break;
        }

        if ( mainData->at(k+1)->id == data->id ){
          setCurrentTab( mainData->at(k)->id );
          break;
        }
      }
    }
    mainData->remove( data );
    repaint();
    barPainter->drawBuffer();
  }
}

void STabBar::setCurrentTab( int id )
{
  TabBarData* data = findData( id );
  if ( data != 0L )
    if ( data->enabled && _currentTab != data->id )
    {
      _currentTab = data->id;
      repaint();

      int curx = 2;
      for ( uint k = 0; k < mainData->count(); k++ ){
        data = mainData->at(k);
        if ( data->id == id ){
          curx += 30;
          break;
        }
        curx += data->width;
      }
      while ( width() - 50 > 30 && ((barPainter->x() + curx < 0) || (barPainter->x() + curx > width() - 50)) ){
        if ( barPainter->x() + curx < 0 )
          rightClicked();
        else
          leftClicked();
      }

      barPainter->drawBuffer();
      emit tabSelected( id );
    }
}

void STabBar::setTabEnabled( int id , bool enabled )
{
  TabBarData* data = findData( id );
  if ( data != 0L )
    if ( data->enabled != enabled )
    {
      data->enabled = enabled;
      if ( _currentTab == data->id )
        for ( uint k = 0; k < mainData->count(); k++ )
          if ( mainData->at(k)->enabled ){
            setCurrentTab( mainData->at(k)->id );
            break;
          }
      barPainter->drawBuffer();
    }
}

bool STabBar::isTabEnabled( int id )
{
  TabBarData* data = findData( id );
  return data == 0L ? false:data->enabled;
}

void STabBar::show()
{
  if ( _currentTab == 0 )
    if ( !mainData->isEmpty() )
      _currentTab = mainData->at(0)->id;

  if ( _currentTab != 0 ){
    setCurrentTab( _currentTab );
  }
  QWidget::show();
}

int STabBar::tabsWidth()
{
  int width = 0;
  for ( uint k = 0; k < mainData->count(); k++ ){
    width += mainData->at(k)->width;
  }
  return width == 0 ? 0:width + 4;
}

void STabBar::fontChange( const QFont & )
{
  if ( iconShow )
    setFixedHeight( fontMetrics().height() + 10 );
  else
    setFixedHeight( fontMetrics().height() + 4 );

  QSize size( width(), height() );
  qApp->notify( this, new QResizeEvent( size , size ) );
  repaint();
}

void STabBar::resizeEvent(QResizeEvent *)
{
  int maxAllowWidth = width() - 50 - barPainter->x();
  barPainter->resize( QMIN(tabsWidth(),maxAllowWidth),  height() - 1 );

  barPainter->drawBuffer();
  left->setGeometry( width()-2*18-2, height()-16-4, 18, 18 );
  right->setGeometry( width()-1*18, height()-16-4, 18, 18 );

  if ( barPainter->x() < 0 ||  tabsWidth() > maxAllowWidth ){
    left->show();
    right->show();
  } else {
    left->hide();
    right->hide();
  }
}

void STabBar::leftClicked()
{
  if ( leftTab != (int)mainData->count() - 1 ){
    int dx = mainData->at( leftTab )->width;
    leftTab++;
    barPainter->move( barPainter->x() - dx, barPainter->y() );
    repaint();
    QSize size( width(), height() );
    qApp->notify( this, new QResizeEvent( size , size ) );
  }
}

void STabBar::rightClicked()
{
  if ( leftTab > 0 ){
    leftTab--;
    int dx = mainData->at( leftTab )->width;
    barPainter->move( barPainter->x() + dx, barPainter->y() );
    repaint();
    QSize size( width(), height() );
    qApp->notify( this, new QResizeEvent( size , size ) );
  }
}




