/***************************************************************************
                         STabBar part of KDEStudio
                             -------------------
    copyright            : (C) 1999 by Judin Maximus
    email                : novaprint@mtu-net.ru
 ***************************************************************************/

#ifndef STABBAR_H
#define STABBAR_H

#include <qwidget.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qlist.h>

#include <qpushbutton.h>
#include <qpopupmenu.h>

struct TabBarData
{
  TabBarData( int _id, const char* _label )
  {
    id = _id;
    width = 0;
    label = _label;
    pix = 0L;
    enabled = true;
  }
  ~TabBarData()
  {
    if ( pix != 0L ) delete pix;
  }
  int      id;
  int      width;
  QString  label;
  QPixmap* pix;
  bool     enabled;
};

class BarPainter;

class STabBar: public QWidget
{Q_OBJECT
friend BarPainter;
public:
  STabBar( QWidget * parent = 0, const char * name = 0 );
  ~STabBar();

  int insertTab( const char* label, int id = -1 );

  void setPixmap( int id, const QPixmap &pix );
  void removeTab( int );
  int  currentTab(){ return _currentTab; }
  void setCurrentTab( int id );

  void setTabEnabled( int, bool );
  bool isTabEnabled( int );

  virtual void show();

  void showTabIcon( bool );

signals:
  void tabSelected( int );
  void rightButtonPress( int, QPoint );

protected slots:
  void leftClicked();
  void rightClicked();

protected:
  virtual void paintEvent(QPaintEvent *);
  virtual void resizeEvent( QResizeEvent * );
  virtual void fontChange( const QFont & );

private:
  TabBarData* findData( int id );
  int tabsWidth();

  BarPainter* barPainter;
  QList<TabBarData> *mainData;
  int _currentTab;
  int leftTab;

  QPushButton *right;
  QPushButton *left;
  bool rightscroll;
  bool leftscroll;
  bool iconShow;
};

class BarPainter: public QWidget
{Q_OBJECT
friend STabBar;
private:
  BarPainter( STabBar * parent );
  ~BarPainter();

  void drawBuffer();
  int findBarByPosX( int );
  QPixmap* buffer;
  int mousePressTab;

protected:
  virtual void mousePressEvent ( QMouseEvent * );
  virtual void mouseReleaseEvent ( QMouseEvent * );

  virtual void resizeEvent( QResizeEvent * );
  virtual void paintEvent( QPaintEvent * );
};

#endif































